@@ -1,7 +1,7 @@
 #! /bin/bash -e
-# $Id: check_jenkins.sh 2878 2013-07-29 17:42:41Z bradbell $
+# $Id: check_jenkins.sh 3788 2016-02-09 15:50:06Z bradbell $
 # -----------------------------------------------------------------------------
-# CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-13 Bradley M. Bell
+# CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 #
 # CppAD is distributed under multiple licenses. This distribution is under
 # the terms of the
@@ -37,3 +37,6 @@ echo_eval cd jenkins
 echo_eval svn update
 # ----------------------------------------------------------------------------
 echo_eval bin/jenkins.sh
+# ----------------------------------------------------------------------------
+echo "$0: OK"
+exit 0
