@@ -1,11 +1,11 @@
-/* $Id: base_adolc.hpp 3495 2014-12-24 01:16:15Z bradbell $ */
-# ifndef CPPAD_BASE_ADOLC_INCLUDED
-# define CPPAD_BASE_ADOLC_INCLUDED
+// $Id: base_adolc.hpp 3804 2016-03-20 15:08:46Z bradbell $
+# ifndef CPPAD_EXAMPLE_BASE_ADOLC_HPP
+# define CPPAD_EXAMPLE_BASE_ADOLC_HPP
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-13 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
-the terms of the 
+the terms of the
                     Eclipse Public License Version 1.0.
 
 A copy of this license is included in the COPYING file of this distribution.
@@ -14,6 +14,17 @@ Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 /*
 $begin base_adolc.hpp$$
 $spell
+	stringstream
+	struct
+	string
+	setprecision
+	str
+	valgrind
+	azmul
+	expm1
+	atanh
+	acosh
+	asinh
 	erf
 	ifndef
 	define
@@ -53,9 +64,6 @@ $spell
 	sqrt
 $$
 
-$index Adolc, adouble as Base$$
-$index Base, Adolc's adouble$$
-$index adouble, as Base$$ 
 
 $section Enable use of AD<Base> where Base is Adolc's adouble Type$$
 
@@ -78,10 +86,10 @@ This file $code base_adolc.hpp$$ requires $code adouble$$ to be defined.
 In addition, it is included before $code <cppad/cppad.hpp>$$,
 but it needs to include parts of CppAD that are used by this file.
 This is done with the following include commands:
-$codep */
+$srccode%cpp% */
 # include <adolc/adolc.h>
 # include <cppad/base_require.hpp>
-/* $$
+/* %$$
 
 $head CondExpOp$$
 The type $code adouble$$ supports a conditional assignment function
@@ -95,9 +103,9 @@ $codei%
 %$$
 This enables one to include conditionals in the recording of
 $code adouble$$ operations and later evaluation for different
-values of the independent variables 
+values of the independent variables
 (in the same spirit as the CppAD $cref CondExp$$ function).
-$codep */
+$srccode%cpp% */
 namespace CppAD {
 	inline adouble CondExpOp(
 		enum  CppAD::CompareOp     cop ,
@@ -139,13 +147,21 @@ namespace CppAD {
 		return result;
 	}
 }
-/* $$
+/* %$$
+
+$head CondExpRel$$
+The $cref/CPPAD_COND_EXP_REL/base_cond_exp/CondExpRel/$$ macro invocation
+$srccode%cpp% */
+namespace CppAD {
+	CPPAD_COND_EXP_REL(adouble)
+}
+/* %$$
 
 $head EqualOpSeq$$
-The Adolc user interface does not specify a way to determine if 
-two $code adouble$$ variables correspond to the same operations sequence. 
+The Adolc user interface does not specify a way to determine if
+two $code adouble$$ variables correspond to the same operations sequence.
 Make $code EqualOpSeq$$ an error if it gets used:
-$codep */
+$srccode%cpp% */
 namespace CppAD {
 	inline bool EqualOpSeq(const adouble &x, const adouble &y)
 	{	CppAD::ErrorHandler::Call(
@@ -156,13 +172,13 @@ namespace CppAD {
 		return false;
 	}
 }
-/* $$
+/* %$$
 
 $head Identical$$
-The Adolc user interface does not specify a way to determine if an 
-$code adouble$$ depends on the independent variables. 
+The Adolc user interface does not specify a way to determine if an
+$code adouble$$ depends on the independent variables.
 To be safe (but slow) return $code false$$ in all the cases below.
-$codep */
+$srccode%cpp% */
 namespace CppAD {
 	inline bool IdenticalPar(const adouble &x)
 	{	return false; }
@@ -173,16 +189,23 @@ namespace CppAD {
 	inline bool IdenticalEqualPar(const adouble &x, const adouble &y)
 	{	return false; }
 }
-/* $$
+/* %$$
 
 $head Integer$$
-$codep */
+$srccode%cpp% */
 	inline int Integer(const adouble &x)
 	{    return static_cast<int>( x.getValue() ); }
-/* $$
+/* %$$
+
+$head azmul$$
+$srccode%cpp% */
+namespace CppAD {
+	CPPAD_AZMUL( adouble )
+}
+/* %$$
 
 $head Ordered$$
-$codep */
+$srccode%cpp% */
 namespace CppAD {
 	inline bool GreaterThanZero(const adouble &x)
 	{    return (x > 0); }
@@ -195,10 +218,10 @@ namespace CppAD {
 	inline bool abs_geq(const adouble& x, const adouble& y)
 	{	return fabs(x) >= fabs(y); }
 }
-/* $$
+/* %$$
 
 $head Unary Standard Math$$
-The following $cref/required/base_require/$$ functions 
+The following $cref/required/base_require/$$ functions
 are defined by the Adolc package for the $code adouble$$ base case:
 $pre
 $$
@@ -215,32 +238,43 @@ $code sinh$$,
 $code sqrt$$,
 $code tan$$.
 
-$head erf$$
-If the error function is supported by the compiler,
-it must also be supported by a $icode Base$$ type;
-see $cref/erf/base_std_math/erf/$$.
-The adolc package does not support this function:
-$codep */
+$head erf, asinh, acosh, atanh, expm1, log1p$$
+If the
+$cref/erf, asinh, acosh, atanh, expm1, log1p
+	/base_std_math
+	/erf, asinh, acosh, atanh, expm1, log1p
+/$$,
+functions are supported by the compiler,
+they must also be supported by a $icode Base$$ type;
+The adolc package does not support these functions so make
+their use an error:
+$srccode%cpp% */
 namespace CppAD {
-# if CPPAD_COMPILER_HAS_ERF
-	inline adouble erf(const adouble& x)
-	{	CPPAD_ASSERT_KNOWN( 
-			false,
-			"erf: adolc does not support the error function"
-		);
-		return 0;
-	}
+# define CPPAD_BASE_ADOLC_NO_SUPPORT(fun)                         \
+    inline adouble fun(const adouble& x)                          \
+    {   CPPAD_ASSERT_KNOWN(                                       \
+            false,                                                \
+            #fun ": adolc does not support this function"         \
+        );                                                        \
+        return 0.0;                                               \
+    }
+# if CPPAD_USE_CPLUSPLUS_2011
+	CPPAD_BASE_ADOLC_NO_SUPPORT(erf)
+	CPPAD_BASE_ADOLC_NO_SUPPORT(asinh)
+	CPPAD_BASE_ADOLC_NO_SUPPORT(acosh)
+	CPPAD_BASE_ADOLC_NO_SUPPORT(atanh)
+	CPPAD_BASE_ADOLC_NO_SUPPORT(expm1)
+	CPPAD_BASE_ADOLC_NO_SUPPORT(log1p)
 # endif
+# undef CPPAD_BASE_ADOLC_NO_SUPPORT
 }
-/* $$ 
-
-
+/* %$$
 
 $head sign$$
-This $cref/required/base_require/$$ function is defined using the 
+This $cref/required/base_require/$$ function is defined using the
 $code codassign$$ function so that its $code adouble$$ operation sequence
 does not depend on the value of $icode x$$.
-$codep */
+$srccode%cpp% */
 namespace CppAD {
 	inline adouble sign(const adouble& x)
 	{	adouble s_plus, s_minus, half(.5);
@@ -252,47 +286,81 @@ namespace CppAD {
 		return s_plus - s_minus;
 	}
 }
-/* $$
+/* %$$
 
 $head abs$$
-This $cref/required/base_require/$$ function uses the adolc $code fabs$$ 
+This $cref/required/base_require/$$ function uses the adolc $code fabs$$
 function:
-$codep */
+$srccode%cpp% */
 namespace CppAD {
 	inline adouble abs(const adouble& x)
 	{	return fabs(x); }
 }
-/* $$
+/* %$$
 
 $head pow$$
-This $cref/required/base_require/$$ function 
+This $cref/required/base_require/$$ function
 is defined by the Adolc package for the $code adouble$$ base case.
 
-$head limits$$
-The following defines the numeric limits functions
-$code epsilon$$, $code min$$, and $code max$$ for the type
-$code adouble$$.
-It also defines the deprecated $code epsilon$$ function:
-$codep */
+$head numeric_limits$$
+The following defines the CppAD $cref numeric_limits$$
+for the type $code adouble$$:
+$srccode%cpp% */
 namespace CppAD {
-	template <>
-	class numeric_limits<adouble> {
-	public:
-		// machine epsilon
-		static adouble epsilon(void)
-		{	return adouble( std::numeric_limits<double>::epsilon() ); }
-		// minimum positive normalized value
-		static adouble min(void)
-		{	return adouble( std::numeric_limits<float>::min() ); }
-		// maximum finite value
-		static adouble max(void)
-		{	return adouble( std::numeric_limits<float>::max() ); }
-	};
-	// deprecated machine epsilon
-	template <> inline adouble epsilon<adouble>(void)
-	{	return numeric_limits<adouble>::epsilon(); }
+	CPPAD_NUMERIC_LIMITS(double, adouble)
 }
-/* $$
+/* %$$
+
+$head to_string$$
+The following defines the CppAD $cref to_string$$ function
+for the type $code adouble$$:
+$srccode%cpp% */
+namespace CppAD {
+	template <> struct to_string_struct<adouble>
+	{	std::string operator()(const adouble& x)
+		{	std::stringstream os;
+			double epsilon    = std::numeric_limits<double>::epsilon();
+			double log10      = std::log( epsilon ) / std::log(10.);
+			size_t n_digits   = 1 - Integer( log10 );
+			os << std::setprecision(n_digits);
+			os << x.value();
+			return os.str();
+		}
+	};
+}
+/* %$$
+
+$head hash_code$$
+It appears that an $code adouble$$ object can have fields
+that are not initialized.
+This results in a $code valgrind$$ error when these fields are used by the
+$cref/default/base_hash/Default/$$ hashing function.
+For this reason, the $code adouble$$ class overrides the default definition.
+$srccode|cpp| */
+namespace CppAD {
+	inline unsigned short hash_code(const adouble& x)
+	{	unsigned short code = 0;
+		double value = x.value();
+		if( value == 0.0 )
+			return code;
+		double log_x = std::log( abs( value ) );
+		// assume log( std::numeric_limits<double>::max() ) is near 700
+		code = static_cast<unsigned short>(
+			(CPPAD_HASH_TABLE_SIZE / 700 + 1) * log_x
+		);
+		code = code % CPPAD_HASH_TABLE_SIZE;
+		return code;
+	}
+}
+/* |$$
+Note that after the hash codes match, the
+$cref/Identical/base_adolc.hpp/Identical/$$ function will be used
+to make sure two values are the same and one can replace the other.
+A more sophisticated implementation of the $code Identical$$ function
+would detect which $code adouble$$ values depend on the
+$code adouble$$ independent variables (and hence can change).
+
+
 $end
 */
 # endif
