@@ -1,11 +1,11 @@
-/* $Id: solve.hpp 2910 2013-10-07 13:27:58Z bradbell $ */
-# ifndef CPPAD_SOLVE_INCLUDED
-# define CPPAD_SOLVE_INCLUDED
+// $Id: solve.hpp 3804 2016-03-20 15:08:46Z bradbell $
+# ifndef CPPAD_IPOPT_SOLVE_HPP
+# define CPPAD_IPOPT_SOLVE_HPP
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-13 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
-the terms of the 
+the terms of the
                     Eclipse Public License Version 1.0.
 
 A copy of this license is included in the COPYING file of this distribution.
@@ -61,29 +61,29 @@ The function $code ipopt::solve$$ solves nonlinear programming
 problems of the form
 $latex \[
 \begin{array}{rll}
-{\rm minimize}      & f (x) 
+{\rm minimize}      & f (x)
 \\
 {\rm subject \; to} & gl \leq g(x) \leq gu
 \\
                     & xl  \leq x   \leq xu
 \end{array}
 \] $$
-This is done using 
+This is done using
 $href%
 	http://www.coin-or.org/projects/Ipopt.xml%
 	Ipopt
 %$$
-optimizer and CppAD for the derivative and sparsity calculations. 
+optimizer and CppAD for the derivative and sparsity calculations.
 
 $head Include File$$
-Currently, this routine 
+Currently, this routine
 $cref/ipopt::solve/ipopt_solve/$$ is not included by the command
 $codei%
 	# include <cppad/cppad.hpp>
 %$$
 (Doing so would require the ipopt library to link
 the corresponding program (even if $code ipopt::solve$$) was not used.)
-For this reason, 
+For this reason,
 if you are using $code ipopt::solve$$ you should use
 $codei%
 	# include <cppad/ipopt/solve.hpp>
@@ -92,12 +92,12 @@ which in turn will also include $code <cppad/cppad.hpp>$$.
 
 $head Bvector$$
 The type $icode Bvector$$ must be a $cref SimpleVector$$ class with
-$cref/elements of type/SimpleVector/Elements of Specified Type/$$ 
+$cref/elements of type/SimpleVector/Elements of Specified Type/$$
 $code bool$$.
 
 $head Dvector$$
 The type $icode DVector$$ must be a $cref SimpleVector$$ class with
-$cref/elements of type/SimpleVector/Elements of Specified Type/$$ 
+$cref/elements of type/SimpleVector/Elements of Specified Type/$$
 $code double$$.
 
 $head options$$
@@ -106,7 +106,7 @@ $codei%
 	const std::string %options%
 %$$
 It contains a list of options.
-Each option, including the last option, 
+Each option, including the last option,
 is terminated by the $code '\n'$$ character.
 Each line consists of two or three tokens separated by one or more spaces.
 
@@ -120,7 +120,7 @@ $cref/operation sequence/glossary/Operation/Sequence/$$ for each
 new value of $icode x$$.
 If the value is $code false$$, $code ipopt::solve$$
 will tape the operation sequence at the value
-of $icode xi$$ and use that sequence for the entire optimization process.  
+of $icode xi$$ and use that sequence for the entire optimization process.
 The default value is $code false$$.
 
 $subhead Sparse$$
@@ -130,7 +130,7 @@ $codei%
 %$$
 If the value is $code true$$, $code ipopt::solve$$ will use a sparse
 matrix representation for the computation of Jacobians and Hessians.
-Otherwise, it will use a full matrix representation for 
+Otherwise, it will use a full matrix representation for
 these calculations.
 The default for $icode value$$ is $code false$$.
 If sparse is true, retape must be false.
@@ -139,23 +139,23 @@ $pre
 $$
 It is unclear if $cref sparse_jacobian$$ would be faster user
 forward or reverse mode so you are able to choose the direction.
-If  
+If
 $codei%
 	%value% == true && %direction% == forward
 %$$
-the Jacobians will be calculated using $code SparseJacobianForward$$. 
-If 
+the Jacobians will be calculated using $code SparseJacobianForward$$.
+If
 $codei%
 	%value% == true && %direction% == reverse
 %$$
-the Jacobians will be calculated using $code SparseJacobianReverse$$. 
+the Jacobians will be calculated using $code SparseJacobianReverse$$.
 
 $subhead String$$
 You can set any Ipopt string option using a line with the following syntax:
 $codei%
 	String %name% %value%
 %$$
-Here $icode name$$ is any valid Ipopt string option 
+Here $icode name$$ is any valid Ipopt string option
 and $icode value$$ is its setting.
 
 $subhead Numeric$$
@@ -163,7 +163,7 @@ You can set any Ipopt numeric option using a line with the following syntax:
 $codei%
 	Numeric %name% %value%
 %$$
-Here $icode name$$ is any valid Ipopt numeric option 
+Here $icode name$$ is any valid Ipopt numeric option
 and $icode value$$ is its setting.
 
 $subhead Integer$$
@@ -171,7 +171,7 @@ You can set any Ipopt integer option using a line with the following syntax:
 $codei%
 	Integer %name% %value%
 %$$
-Here $icode name$$ is any valid Ipopt integer option 
+Here $icode name$$ is any valid Ipopt integer option
 and $icode value$$ is its setting.
 
 $head xi$$
@@ -230,7 +230,7 @@ and the arguments to $icode fg$$, $icode x$$ have the following meaning:
 
 $subhead ADvector$$
 The type $icode%FG_eval%::ADvector%$$ must be a $cref SimpleVector$$ class with
-$cref/elements of type/SimpleVector/Elements of Specified Type/$$ 
+$cref/elements of type/SimpleVector/Elements of Specified Type/$$
 $code AD<double>$$.
 
 $subhead x$$
@@ -268,7 +268,7 @@ The $icode status$$ field of $icode solution$$ has prototype
 $codei%
 	ipopt::solve_result<%Dvector%>::status_type %solution%.status
 %$$
-It is the final Ipopt status for the optimizer. 
+It is the final Ipopt status for the optimizer.
 Here is a list of the possible values for the status:
 
 $table
@@ -282,7 +282,7 @@ The status returned by the optimizer is not defined in the Ipopt
 documentation for $code finalize_solution$$.
 $rnext
 success $cnext
-Algorithm terminated successfully at a point satisfying the convergence 
+Algorithm terminated successfully at a point satisfying the convergence
 tolerances (see Ipopt options).
 $rnext
 maxiter_exceeded $cnext
@@ -292,8 +292,8 @@ stop_at_tiny_step $cnext
 Algorithm terminated because progress was very slow.
 $rnext
 stop_at_acceptable_point $cnext
-Algorithm stopped at a point that was converged, 
-not to the 'desired' tolerances, but to 'acceptable' tolerances 
+Algorithm stopped at a point that was converged,
+not to the 'desired' tolerances, but to 'acceptable' tolerances
 (see Ipopt options).
 $rnext
 local_infeasibility $cnext
@@ -310,11 +310,11 @@ restoration_failure $cnext
 Restoration phase failed, algorithm doesn't know how to proceed.
 $rnext
 error_in_step_computation $cnext
-An unrecoverable error occurred while Ipopt tried to 
+An unrecoverable error occurred while Ipopt tried to
 compute the search direction.
 $rnext
 invalid_number_detected $cnext
-Algorithm received an invalid number (such as $code nan$$ or $code inf$$) 
+Algorithm received an invalid number (such as $code nan$$ or $code inf$$)
 from the users function $icode%fg_info%.eval%$$ or from the CppAD evaluations
 of its derivatives
 (see the Ipopt option $code check_derivatives_for_naninf$$).
@@ -338,7 +338,7 @@ $codei%
 	%Vector% %solution%.zl
 %$$
 and its size is equal to $icode nx$$.
-It is the final Lagrange multipliers for the 
+It is the final Lagrange multipliers for the
 lower bounds on $latex x$$.
 
 $subhead zu$$
@@ -347,7 +347,7 @@ $codei%
 	%Vector% %solution%.zu
 %$$
 and its size is equal to $icode nx$$.
-It is the final Lagrange multipliers for the 
+It is the final Lagrange multipliers for the
 upper bounds on $latex x$$.
 
 $subhead g$$
@@ -364,7 +364,7 @@ $codei%
 	%Vector%> %solution%.lambda
 %$$
 and its size is equal to $icode ng$$.
-It is the final value for the 
+It is the final value for the
 Lagrange multipliers corresponding to the constraint function.
 
 $subhead obj_value$$
@@ -398,7 +398,7 @@ $subhead ode_inverse$$
 The file
 $cref%example/ipopt_solve/ode_inverse.cpp%ipopt_solve_ode_inverse.cpp%$$
 demonstrates using Ipopt to solve for parameters in an ODE model.
-	
+
 $end
 -------------------------------------------------------------------------------
 */
@@ -407,10 +407,8 @@ $end
 namespace CppAD { // BEGIN_CPPAD_NAMESPACE
 namespace ipopt {
 /*!
-\defgroup solve_hpp solve.hpp
-\{
 \file solve.hpp
-\brief Implement the ipopt::solve Nonlinear Programming Solver 
+\brief Implement the ipopt::solve Nonlinear Programming Solver
 */
 
 /*!
@@ -431,7 +429,7 @@ It must also support
 to dentify the type used for the arguments to fg_eval.
 
 \param options
-list of options, one for each line. 
+list of options, one for each line.
 Ipopt options (are optional) and have one of the following forms
 \code
 	String   name  value
@@ -471,14 +469,14 @@ structure that holds the solution of the optimization.
 template <class Dvector, class FG_eval>
 void solve(
 	const std::string&                   options   ,
-	const Dvector&                       xi        , 
+	const Dvector&                       xi        ,
 	const Dvector&                       xl        ,
-	const Dvector&                       xu        , 
-	const Dvector&                       gl        , 
-	const Dvector&                       gu        , 
-	FG_eval&                             fg_eval   , 
+	const Dvector&                       xu        ,
+	const Dvector&                       gl        ,
+	const Dvector&                       gu        ,
+	FG_eval&                             fg_eval   ,
 	ipopt::solve_result<Dvector>&        solution  )
-{ 	bool ok = true;
+{	bool ok = true;
 
 	typedef typename FG_eval::ADvector ADvector;
 
@@ -504,7 +502,7 @@ void solve(
 	bool sparse_forward  = false;
 	bool sparse_reverse  = false;
 	while( begin_1 < options.size() )
-	{ 	// split this line into tokens
+	{	// split this line into tokens
 		while( options[begin_1] == ' ')
 			begin_1++;
 		end_1   = options.find_first_of(" \n", begin_1);
@@ -518,13 +516,13 @@ void solve(
 		end_3   = options.find_first_of(" \n", begin_3);
 
 		// check for errors
-		CPPAD_ASSERT_KNOWN( 
-			(end_1 != std::string::npos)  & 
-			(end_2 != std::string::npos)  & 
+		CPPAD_ASSERT_KNOWN(
+			(end_1 != std::string::npos)  &
+			(end_2 != std::string::npos)  &
 			(end_3 != std::string::npos)  ,
 			"ipopt::solve: missing '\\n' at end of an option line"
 		);
-		CPPAD_ASSERT_KNOWN( 
+		CPPAD_ASSERT_KNOWN(
 			(end_1 > begin_1) & (end_2 > begin_2) ,
 			"ipopt::solve: an option line does not have two tokens"
 		);
@@ -540,8 +538,8 @@ void solve(
 		three_tok |= tok_1 == "String";
 		three_tok |= tok_1 == "Numeric";
 		three_tok |= tok_1 == "Integer";
-		if( three_tok ) 
-		{	CPPAD_ASSERT_KNOWN( 
+		if( three_tok )
+		{	CPPAD_ASSERT_KNOWN(
 				(end_3 > begin_3) ,
 				"ipopt::solve: a Sparse, String, Numeric, or Integer\n"
 				"option line does not have three tokens."
@@ -578,11 +576,11 @@ void solve(
 		else if ( tok_1 == "String" )
 			app->Options()->SetStringValue(tok_2.c_str(), tok_3.c_str());
 		else if ( tok_1 == "Numeric" )
-		{	Ipopt::Number value = std::atof( tok_3.c_str() );	
+		{	Ipopt::Number value = std::atof( tok_3.c_str() );
 			app->Options()->SetNumericValue(tok_2.c_str(), value);
 		}
 		else if ( tok_1 == "Integer" )
-		{	Ipopt::Index value = std::atoi( tok_3.c_str() );	
+		{	Ipopt::Index value = std::atoi( tok_3.c_str() );
 			app->Options()->SetIntegerValue(tok_2.c_str(), value);
 		}
 		else	CPPAD_ASSERT_KNOWN(
@@ -601,7 +599,7 @@ void solve(
 		);
 		begin_1++;
 	}
-	CPPAD_ASSERT_KNOWN( 
+	CPPAD_ASSERT_KNOWN(
 		! ( retape & (sparse_forward | sparse_reverse) ) ,
 		"ipopt::solve: retape and sparse both true is not supported."
 	);
@@ -610,27 +608,27 @@ void solve(
 	Ipopt::ApplicationReturnStatus status = app->Initialize();
 	ok    &= status == Ipopt::Solve_Succeeded;
 	if( ! ok )
-	{	solution.status = solve_result<Dvector>::unknown; 
+	{	solution.status = solve_result<Dvector>::unknown;
 		return;
 	}
 
 	// Create an interface from Ipopt to this specific problem.
 	// Note the assumption here that ADvector is same as cppd_ipopt::ADvector
 	size_t nf = 1;
-	Ipopt::SmartPtr<Ipopt::TNLP> cppad_nlp = 
+	Ipopt::SmartPtr<Ipopt::TNLP> cppad_nlp =
 	new CppAD::ipopt::solve_callback<Dvector, ADvector, FG_eval>(
-		nf, 
-		nx, 
-		ng, 
-		xi, 
-		xl, 
-		xu, 
-		gl, 
-		gu, 
-		fg_eval, 
-		retape, 
-		sparse_forward, 
-		sparse_reverse, 
+		nf,
+		nx,
+		ng,
+		xi,
+		xl,
+		xu,
+		gl,
+		gu,
+		fg_eval,
+		retape,
+		sparse_forward,
+		sparse_reverse,
 		solution
 	);
 
@@ -640,7 +638,6 @@ void solve(
 	return;
 }
 
-/*! \} */
 } // end ipopt namespace
 } // END_CPPAD_NAMESPACE
 # endif
