@@ -1,29 +1,27 @@
-/* $Id: rev_hes_sweep.hpp 2991 2013-10-22 16:25:15Z bradbell $ */
-# ifndef CPPAD_REV_HES_SWEEP_INCLUDED
-# define CPPAD_REV_HES_SWEEP_INCLUDED
+// $Id: rev_hes_sweep.hpp 3853 2016-12-14 14:40:11Z bradbell $
+# ifndef CPPAD_LOCAL_REV_HES_SWEEP_HPP
+# define CPPAD_LOCAL_REV_HES_SWEEP_HPP
 
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-13 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
-the terms of the 
+the terms of the
                     Eclipse Public License Version 1.0.
 
 A copy of this license is included in the COPYING file of this distribution.
 Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 -------------------------------------------------------------------------- */
 
-namespace CppAD { // BEGIN_CPPAD_NAMESPACE
+namespace CppAD { namespace local { // BEGIN_CPPAD_LOCAL_NAMESPACE
 /*!
-\defgroup rev_hes_sweep_hpp rev_hes_sweep.hpp
-\{
 \file rev_hes_sweep.hpp
 Compute Reverse mode Hessian sparsity patterns.
 */
 
 /*!
 \def CPPAD_REV_HES_SWEEP_TRACE
-This value is either zero or one. 
+This value is either zero or one.
 Zero is the normal operational value.
 If it is one, a trace of every rev_hes_sweep computation is printed.
 */
@@ -32,25 +30,25 @@ If it is one, a trace of every rev_hes_sweep computation is printed.
 /*!
 Given the forward Jacobian sparsity pattern for all the variables,
 and the reverse Jacobian sparsity pattern for the dependent variables,
-RevHesSweep computes the Hessian sparsity pattern for all the independent 
+RevHesSweep computes the Hessian sparsity pattern for all the independent
 variables.
 
 \tparam Base
 base type for the operator; i.e., this operation sequence was recorded
-using AD< \a Base > and computations by this routine are done using type 
+using AD< \a Base > and computations by this routine are done using type
 \a Base.
 
 \tparam Vector_set
 is the type used for vectors of sets. It can be either
-\c sparse_pack, \c sparse_set, or \c sparse_list.
+sparse_pack or sparse_list.
 
 \param n
 is the number of independent variables on the tape.
 
 \param numvar
 is the total number of variables on the tape; i.e.,
-\a play->num_rec_var().
-This is also the number of rows in the entire sparsity pattern 
+\a play->num_var_rec().
+This is also the number of rows in the entire sparsity pattern
 \a rev_hes_sparse.
 
 \param play
@@ -63,21 +61,21 @@ where \f$ n \f$ is the number of independent variables
 and \f$ m \f$ is the number of dependent variables.
 The object \a play is effectly constant.
 It is not declared const because while playing back the tape
-the object \a play holds information about the currentl location
+the object \a play holds information about the current location
 with in the tape and this changes during playback.
 
 \param for_jac_sparse
-For i = 0 , ... , \a numvar - 1, 
+For i = 0 , ... , \a numvar - 1,
 (for all the variables on the tape),
 the forward Jacobian sparsity pattern for the variable with index i
 corresponds to the set with index i in \a for_jac_sparse.
 
 \param RevJac
 \b Input:
-For i = 0, ... , \a numvar - 1 
+For i = 0, ... , \a numvar - 1
 the if the variable with index i on the tape is an dependent variable and
 included in the Hessian, \a RevJac[ i ] is equal to true,
-otherwise it is equal to false. 
+otherwise it is equal to false.
 \n
 \n
 \b Output: The values in \a RevJac upon return are not specified; i.e.,
@@ -88,14 +86,14 @@ The reverse Hessian sparsity pattern for the variable with index i
 corresponds to the set with index i in \a rev_hes_sparse.
 \n
 \n
-\b Input: For i = 0 , ... , \a numvar - 1  
+\b Input: For i = 0 , ... , \a numvar - 1
 the reverse Hessian sparsity pattern for the variable with index i is empty.
 \n
 \n
 \b Output: For j = 1 , ... , \a n,
-the reverse Hessian sparsity pattern for the independent dependent variable 
+the reverse Hessian sparsity pattern for the independent dependent variable
 with index (j-1) is given by the set with index j
-in \a rev_hes_sparse. 
+in \a rev_hes_sparse.
 The values in the rest of \a rev_hes_sparse are not specified; i.e.,
 they are used for temporary work space.
 */
@@ -104,8 +102,8 @@ template <class Base, class Vector_set>
 void RevHesSweep(
 	size_t                n,
 	size_t                numvar,
-	player<Base>         *play,
-	Vector_set&           for_jac_sparse, // should be const
+	local::player<Base>*         play,
+	const Vector_set&     for_jac_sparse,
 	bool*                 RevJac,
 	Vector_set&           rev_hes_sparse
 )
@@ -117,12 +115,12 @@ void RevHesSweep(
 	const addr_t*   arg = CPPAD_NULL;
 
 	// length of the parameter vector (used by CppAD assert macros)
-	const size_t num_par = play->num_rec_par();
+	const size_t num_par = play->num_par_rec();
 
 	size_t             i, j, k;
 
 	// check numvar argument
-	CPPAD_ASSERT_UNKNOWN( play->num_rec_var()     == numvar );
+	CPPAD_ASSERT_UNKNOWN( play->num_var_rec()    == numvar );
 	CPPAD_ASSERT_UNKNOWN( for_jac_sparse.n_set() == numvar );
 	CPPAD_ASSERT_UNKNOWN( rev_hes_sparse.n_set() == numvar );
 	CPPAD_ASSERT_UNKNOWN( numvar > 0 );
@@ -132,15 +130,15 @@ void RevHesSweep(
 	CPPAD_ASSERT_UNKNOWN( for_jac_sparse.end() == limit );
 
 	// check number of sets match
-	CPPAD_ASSERT_UNKNOWN( 
+	CPPAD_ASSERT_UNKNOWN(
 		for_jac_sparse.n_set() == rev_hes_sparse.n_set()
 	);
 
 	// vecad_sparsity contains a sparsity pattern for each VecAD object.
-	// vecad_ind maps a VecAD index (beginning of the VecAD object) 
+	// vecad_ind maps a VecAD index (beginning of the VecAD object)
 	// to the index for the corresponding set in vecad_sparsity.
-	size_t num_vecad_ind   = play->num_rec_vecad_ind();
-	size_t num_vecad_vec   = play->num_rec_vecad_vec();
+	size_t num_vecad_ind   = play->num_vec_ind_rec();
+	size_t num_vecad_vec   = play->num_vecad_vec_rec();
 	Vector_set vecad_sparse;
 	vecad_sparse.resize(num_vecad_vec, limit);
 	pod_vector<size_t> vecad_ind;
@@ -154,20 +152,22 @@ void RevHesSweep(
 		{	// length of this VecAD
 			length   = play->GetVecInd(j);
 			// set vecad_ind to proper index for this VecAD
-			vecad_ind[j] = i; 
+			vecad_ind[j] = i;
 			// make all other values for this vector invalid
 			for(k = 1; k <= length; k++)
 				vecad_ind[j+k] = num_vecad_vec;
 			// start of next VecAD
 			j       += length + 1;
-			// initialize this vector's reverse jacobian value 
+			// initialize this vector's reverse jacobian value
 			vecad_jac[i] = false;
 		}
-		CPPAD_ASSERT_UNKNOWN( j == play->num_rec_vecad_ind() );
+		CPPAD_ASSERT_UNKNOWN( j == play->num_vec_ind_rec() );
 	}
 
 	// work space used by UserOp.
+	vector<Base>       user_x;   // parameters in x as integers;
 	vector<size_t>     user_ix;  // variable indices for argument vector x
+	//
 	typedef std::set<size_t> size_set;
 	size_set::iterator set_itr;  // iterator for a standard set
 	size_set::iterator set_end;  // end of iterator sequence
@@ -179,28 +179,33 @@ void RevHesSweep(
 	vector<bool>       bool_u;   // bool reverse Hessian sparsity for y
 	vector<bool>       bool_v;   // bool reverse Hessian sparsity for x
 	//
+	vectorBool         pack_r;   // pack forward Jacobian sparsity for x
+	vectorBool         pack_u;   // pack reverse Hessian sparsity for y
+	vectorBool         pack_v;   // pack reverse Hessian sparsity for x
+	//
 	vector<bool>       user_vx;  // which components of x are variables
 	vector<bool>       user_s;   // reverse Jacobian sparsity for y
 	vector<bool>       user_t;   // reverse Jacobian sparsity for x
 	const size_t user_q = limit; // maximum element plus one
-	size_t user_index = 0;       // indentifier for this atomic operation
-	size_t user_id    = 0;       // user identifier for this call to operator
-	size_t user_i     = 0;       // index in result vector
-	size_t user_j     = 0;       // index in argument vector
-	size_t user_m     = 0;       // size of result vector
-	size_t user_n     = 0;       // size of arugment vector
 	//
 	atomic_base<Base>* user_atom = CPPAD_NULL; // user's atomic op calculator
-	bool               user_bool = false;      // use bool or set sparsity ?
-# ifndef NDEBUG
+	bool               user_pack = false;      // sparsity pattern type is pack
+	bool               user_bool = false;      // sparsity pattern type is bool
+	bool               user_set  = false;      // sparsity pattern type is set
 	bool               user_ok   = false;      // atomic op return value
-# endif
-	// next expected operator in a UserOp sequence
-	enum { user_start, user_arg, user_ret, user_end } user_state = user_end;
-
-
+	//
+	// information defined by forward_user
+	size_t user_old=0, user_m=0, user_n=0, user_i=0, user_j=0;
+	enum_user_state user_state = end_user; // proper initialization
+	//
+	// pointer to the beginning of the parameter vector
+	// (used by atomic functions
+	const Base* parameter = CPPAD_NULL;
+	if( num_par > 0 )
+		parameter = play->GetPar();
+	//
 	// Initialize
-	play->start_reverse(op, arg, i_op, i_var);
+	play->reverse_start(op, arg, i_op, i_var);
 	CPPAD_ASSERT_UNKNOWN( op == EndOp );
 # if CPPAD_REV_HES_SWEEP_TRACE
 	std::cout << std::endl;
@@ -209,9 +214,10 @@ void RevHesSweep(
 # endif
 	bool more_operators = true;
 	while(more_operators)
-	{
+	{	bool flag; // temporary for use in switch cases
+		//
 		// next op
-		play->next_reverse(op, arg, i_op, i_var);
+		play->reverse_next(op, arg, i_op, i_var);
 # ifndef NDEBUG
 		if( i_op <= n )
 		{	CPPAD_ASSERT_UNKNOWN((op == InvOp) | (op == BeginOp));
@@ -255,6 +261,17 @@ void RevHesSweep(
 			break;
 			// -------------------------------------------------
 
+# if CPPAD_USE_CPLUSPLUS_2011
+			case AcoshOp:
+			// sqrt(x * x - 1), acosh(x)
+			CPPAD_ASSERT_NARG_NRES(op, 1, 2)
+			reverse_sparse_hessian_nonlinear_unary_op(
+			i_var, arg[0], RevJac, for_jac_sparse, rev_hes_sparse
+			);
+			break;
+# endif
+			// -------------------------------------------------
+
 			case AsinOp:
 			// sqrt(1 - x * x), asin(x)
 			CPPAD_ASSERT_NARG_NRES(op, 1, 2)
@@ -264,6 +281,17 @@ void RevHesSweep(
 			break;
 			// -------------------------------------------------
 
+# if CPPAD_USE_CPLUSPLUS_2011
+			case AsinhOp:
+			// sqrt(1 + x * x), asinh(x)
+			CPPAD_ASSERT_NARG_NRES(op, 1, 2)
+			reverse_sparse_hessian_nonlinear_unary_op(
+			i_var, arg[0], RevJac, for_jac_sparse, rev_hes_sparse
+			);
+			break;
+# endif
+			// -------------------------------------------------
+
 			case AtanOp:
 			// 1 + x * x, atan(x)
 			CPPAD_ASSERT_NARG_NRES(op, 1, 2)
@@ -273,6 +301,17 @@ void RevHesSweep(
 			break;
 			// -------------------------------------------------
 
+# if CPPAD_USE_CPLUSPLUS_2011
+			case AtanhOp:
+			// 1 - x * x, atanh(x)
+			CPPAD_ASSERT_NARG_NRES(op, 1, 2)
+			reverse_sparse_hessian_nonlinear_unary_op(
+			i_var, arg[0], RevJac, for_jac_sparse, rev_hes_sparse
+			);
+			break;
+# endif
+			// -------------------------------------------------
+
 			case BeginOp:
 			CPPAD_ASSERT_NARG_NRES(op, 1, 1)
 			more_operators = false;
@@ -281,16 +320,16 @@ void RevHesSweep(
 
 			case CSkipOp:
 			// CSkipOp has a variable number of arguments and
-			// next_reverse thinks it one has one argument.
-			// We must inform next_reverse of this special case.
+			// reverse_next thinks it one has one argument.
+			// We must inform reverse_next of this special case.
 			play->reverse_cskip(op, arg, i_op, i_var);
 			break;
 			// -------------------------------------------------
 
 			case CSumOp:
 			// CSumOp has a variable number of arguments and
-			// next_reverse thinks it one has one argument.
-			// We must inform next_reverse of this special case.
+			// reverse_next thinks it one has one argument.
+			// We must inform reverse_next of this special case.
 			play->reverse_csum(op, arg, i_op, i_var);
 			reverse_sparse_hessian_csum_op(
 				i_var, arg, RevJac, rev_hes_sparse
@@ -305,12 +344,6 @@ void RevHesSweep(
 			break;
 			// ---------------------------------------------------
 
-			case ComOp:
-			CPPAD_ASSERT_NARG_NRES(op, 4, 0)
-			CPPAD_ASSERT_UNKNOWN( arg[1] > 1 );
-			break;
-			// --------------------------------------------------
-
 			case CosOp:
 			// sin(x), cos(x)
 			CPPAD_ASSERT_NARG_NRES(op, 1, 2)
@@ -359,6 +392,16 @@ void RevHesSweep(
 			break;
 			// -------------------------------------------------
 
+			case ErfOp:
+			// arg[1] is always the parameter 0
+			// arg[2] is always the parameter 2 / sqrt(pi)
+			CPPAD_ASSERT_NARG_NRES(op, 3, 5);
+			reverse_sparse_hessian_nonlinear_unary_op(
+			i_var, arg[0], RevJac, for_jac_sparse, rev_hes_sparse
+			);
+			break;
+			// -------------------------------------------------
+
 			case ExpOp:
 			CPPAD_ASSERT_NARG_NRES(op, 1, 1)
 			reverse_sparse_hessian_nonlinear_unary_op(
@@ -367,6 +410,16 @@ void RevHesSweep(
 			break;
 			// -------------------------------------------------
 
+# if CPPAD_USE_CPLUSPLUS_2011
+			case Expm1Op:
+			CPPAD_ASSERT_NARG_NRES(op, 1, 1)
+			reverse_sparse_hessian_nonlinear_unary_op(
+			i_var, arg[0], RevJac, for_jac_sparse, rev_hes_sparse
+			);
+			break;
+# endif
+			// -------------------------------------------------
+
 			case InvOp:
 			CPPAD_ASSERT_NARG_NRES(op, 0, 1)
 			// Z is already defined
@@ -403,6 +456,20 @@ void RevHesSweep(
 			break;
 			// -------------------------------------------------
 
+			case EqpvOp:
+			case EqvvOp:
+			case LtpvOp:
+			case LtvpOp:
+			case LtvvOp:
+			case LepvOp:
+			case LevpOp:
+			case LevvOp:
+			case NepvOp:
+			case NevvOp:
+			CPPAD_ASSERT_NARG_NRES(op, 2, 0);
+			break;
+			// -------------------------------------------------
+
 			case LogOp:
 			CPPAD_ASSERT_NARG_NRES(op, 1, 1)
 			reverse_sparse_hessian_nonlinear_unary_op(
@@ -411,12 +478,14 @@ void RevHesSweep(
 			break;
 			// -------------------------------------------------
 
-			case MulvvOp:
-			CPPAD_ASSERT_NARG_NRES(op, 2, 1)
-			reverse_sparse_hessian_mul_op(
-			i_var, arg, RevJac, for_jac_sparse, rev_hes_sparse
+# if CPPAD_USE_CPLUSPLUS_2011
+			case Log1pOp:
+			CPPAD_ASSERT_NARG_NRES(op, 1, 1)
+			reverse_sparse_hessian_nonlinear_unary_op(
+			i_var, arg[0], RevJac, for_jac_sparse, rev_hes_sparse
 			);
 			break;
+# endif
 			// -------------------------------------------------
 
 			case MulpvOp:
@@ -427,6 +496,14 @@ void RevHesSweep(
 			break;
 			// -------------------------------------------------
 
+			case MulvvOp:
+			CPPAD_ASSERT_NARG_NRES(op, 2, 1)
+			reverse_sparse_hessian_mul_op(
+			i_var, arg, RevJac, for_jac_sparse, rev_hes_sparse
+			);
+			break;
+			// -------------------------------------------------
+
 			case ParOp:
 			CPPAD_ASSERT_NARG_NRES(op, 1, 1)
 
@@ -451,7 +528,7 @@ void RevHesSweep(
 
 			case PowvvOp:
 			CPPAD_ASSERT_NARG_NRES(op, 2, 3)
-                        reverse_sparse_hessian_pow_op(
+			reverse_sparse_hessian_pow_op(
 			i_var, arg, RevJac, for_jac_sparse, rev_hes_sparse
 			);
 			break;
@@ -578,24 +655,19 @@ void RevHesSweep(
 
 			case UserOp:
 			// start or end an atomic operation sequence
-			CPPAD_ASSERT_UNKNOWN( NumRes( UserOp ) == 0 );
-			CPPAD_ASSERT_UNKNOWN( NumArg( UserOp ) == 4 );
-			if( user_state == user_end )
-			{	user_index = arg[0];
-				user_id    = arg[1];
-				user_n     = arg[2];
-				user_m     = arg[3];
-				user_atom  = atomic_base<Base>::class_object(user_index);
-# ifndef NDEBUG
-				if( user_atom == CPPAD_NULL )
-				{	std::string msg = 
-						atomic_base<Base>::class_name(user_index)
-						+ ": atomic_base function has been deleted";
-					CPPAD_ASSERT_KNOWN(false, msg.c_str() );
-				}
-# endif
+			flag = user_state == end_user;
+			user_atom = play->reverse_user(op, user_state,
+				user_old, user_m, user_n, user_i, user_j
+			);
+			if( flag )
+			{	user_pack  = user_atom->sparsity() ==
+							atomic_base<Base>::pack_sparsity_enum;
 				user_bool  = user_atom->sparsity() ==
 							atomic_base<Base>::bool_sparsity_enum;
+				user_set   = user_atom->sparsity() ==
+							atomic_base<Base>::set_sparsity_enum;
+				CPPAD_ASSERT_UNKNOWN( user_pack || user_bool || user_set );
+				user_x.resize(user_n);
 				user_ix.resize(user_n);
 				user_vx.resize(user_n);
 				user_s.resize(user_m);
@@ -606,6 +678,24 @@ void RevHesSweep(
 					user_s[i] = false;
 				for(i = 0; i < user_n; i++)
 					user_t[i] = false;
+				if( user_pack )
+				{	pack_r.resize(user_n * user_q);
+					pack_u.resize(user_m * user_q);
+					pack_v.resize(user_n * user_q);
+					// simpler to initialize all patterns as empty
+					for(i = 0; i < user_m; i++)
+					{
+						for(j = 0; j < user_q; j++)
+							pack_u[ i * user_q + j] = false;
+					}
+					for(i = 0; i < user_n; i++)
+					{
+						for(j = 0; j < user_q; j++)
+						{	pack_r[ i * user_q + j] = false;
+							pack_v[ i * user_q + j] = false;
+						}
+					}
+				}
 				if( user_bool )
 				{	bool_r.resize(user_n * user_q);
 					bool_u.resize(user_m * user_q);
@@ -624,7 +714,7 @@ void RevHesSweep(
 						}
 					}
 				}
-				else
+				if( user_set )
 				{	set_r.resize(user_n);
 					set_u.resize(user_m);
 					set_v.resize(user_n);
@@ -635,57 +725,64 @@ void RevHesSweep(
 						set_v[i].clear();
 					}
 				}
-				user_j     = user_n;
-				user_i     = user_m;
-				user_state = user_ret;
 			}
 			else
-			{	CPPAD_ASSERT_UNKNOWN( user_state == user_start );
-				CPPAD_ASSERT_UNKNOWN( user_index == size_t(arg[0]) );
-				CPPAD_ASSERT_UNKNOWN( user_id    == size_t(arg[1]) );
-				CPPAD_ASSERT_UNKNOWN( user_n     == size_t(arg[2]) );
-				CPPAD_ASSERT_UNKNOWN( user_m     == size_t(arg[3]) );
-				user_state = user_end;
-
-				// call users function for this operation
-				user_atom->set_id(user_id);
-# ifdef NDEBUG
-			 	if( user_bool )
-					user_atom->rev_sparse_hes(user_vx,
-						user_s, user_t, user_q, bool_r, bool_u, bool_v
-				);
-				else
-					user_atom->rev_sparse_hes(user_vx,
-						user_s, user_t, user_q, set_r, set_u, set_v
-				);
-# else
-			 	if( user_bool )
-					user_ok = user_atom->rev_sparse_hes(user_vx,
-						user_s, user_t, user_q, bool_r, bool_u, bool_v
-				);
-				else
-					user_ok = user_atom->rev_sparse_hes(user_vx,
-						user_s, user_t, user_q, set_r, set_u, set_v
-				);
+			{	// call users function for this operation
+				user_atom->set_old(user_old);
+				if( user_pack )
+				{	user_ok = user_atom->rev_sparse_hes(user_vx,
+						user_s, user_t, user_q, pack_r, pack_u, pack_v, user_x
+					);
+					if( ! user_ok ) user_ok = user_atom->rev_sparse_hes(
+						user_vx, user_s, user_t, user_q, pack_r, pack_u, pack_v
+					);
+				}
+				if( user_bool )
+				{	user_ok = user_atom->rev_sparse_hes(user_vx,
+						user_s, user_t, user_q, bool_r, bool_u, bool_v, user_x
+					);
+					if( ! user_ok ) user_ok = user_atom->rev_sparse_hes(
+						user_vx, user_s, user_t, user_q, bool_r, bool_u, bool_v
+					);
+				}
+				if( user_set )
+				{	user_ok = user_atom->rev_sparse_hes(user_vx,
+						user_s, user_t, user_q, set_r, set_u, set_v, user_x
+					);
+					if( ! user_ok ) user_ok = user_atom->rev_sparse_hes(
+						user_vx, user_s, user_t, user_q, set_r, set_u, set_v
+					);
+				}
 				if( ! user_ok )
-				{	std::string msg = 
-						atomic_base<Base>::class_name(user_index)
-						+ ": atomic_base.rev_sparse_hes: returned false";
+				{	std::string msg =
+						user_atom->afun_name()
+						+ ": atomic_base.rev_sparse_hes: returned false\n";
+					if( user_pack )
+						msg += "sparsity = pack_sparsity_enum";
+					if( user_bool )
+						msg += "sparsity = bool_sparsity_enum";
+					if( user_set )
+						msg += "sparsity = set_sparsity_enum";
 					CPPAD_ASSERT_KNOWN(false, msg.c_str() );
 				}
-# endif
 				for(i = 0; i < user_n; i++) if( user_ix[i] > 0 )
 				{
 					size_t  i_x = user_ix[i];
 					if( user_t[i] )
 						RevJac[i_x] = true;
+					if( user_pack )
+					{
+						for(j = 0; j < user_q; j++)
+							if( pack_v[ i * user_q + j ] )
+								rev_hes_sparse.add_element(i_x, j);
+					}
 					if( user_bool )
 					{
 						for(j = 0; j < user_q; j++)
 							if( bool_v[ i * user_q + j ] )
 								rev_hes_sparse.add_element(i_x, j);
 					}
-					else
+					if( user_set )
 					{
 						set_itr = set_v[i].begin();
 						set_end = set_v[i].end();
@@ -693,76 +790,103 @@ void RevHesSweep(
 							rev_hes_sparse.add_element(i_x, *set_itr++);
 					}
 				}
-               }
+			}
 			break;
 
 			case UsrapOp:
 			// parameter argument in an atomic operation sequence
-			CPPAD_ASSERT_UNKNOWN( user_state == user_arg );
-			CPPAD_ASSERT_UNKNOWN( 0 < user_j && user_j <= user_n );
-			CPPAD_ASSERT_UNKNOWN( NumArg(op) == 1 );
 			CPPAD_ASSERT_UNKNOWN( size_t(arg[0]) < num_par );
-			--user_j;
+			play->reverse_user(op, user_state,
+				user_old, user_m, user_n, user_i, user_j
+			);
 			user_ix[user_j] = 0;
 			user_vx[user_j] = false;
-			if( user_j == 0 )
-				user_state = user_start;
+			//
+			// parameters as integers
+			user_x[user_j] = parameter[arg[0]];
+			//
 			break;
 
 			case UsravOp:
 			// variable argument in an atomic operation sequence
-			CPPAD_ASSERT_UNKNOWN( user_state == user_arg );
-			CPPAD_ASSERT_UNKNOWN( 0 < user_j && user_j <= user_n );
-			CPPAD_ASSERT_UNKNOWN( NumArg(op) == 1 );
 			CPPAD_ASSERT_UNKNOWN( size_t(arg[0]) <= i_var );
 			CPPAD_ASSERT_UNKNOWN( 0 < arg[0] );
-			--user_j;
+			play->reverse_user(op, user_state,
+				user_old, user_m, user_n, user_i, user_j
+			);
 			user_ix[user_j] = arg[0];
 			user_vx[user_j] = true;
-			for_jac_sparse.begin(arg[0]);
-			i = for_jac_sparse.next_element();
-			while( i < user_q )
-			{	if( user_bool )
-					bool_r[ user_j * user_q + i ] = true;
-				else
-					set_r[user_j].insert(i);
-				i = for_jac_sparse.next_element();
+			//
+			// variable as integers
+			user_x[user_j] = CppAD::numeric_limits<Base>::quiet_NaN();
+			//
+			{	typename Vector_set::const_iterator itr(for_jac_sparse, arg[0]);
+				i = *itr;
+				while( i < user_q )
+				{	if( user_pack )
+						pack_r[ user_j * user_q + i ] = true;
+					if( user_bool )
+						bool_r[ user_j * user_q + i ] = true;
+					if( user_set )
+						set_r[user_j].insert(i);
+					i = *(++itr);
+				}
 			}
-			if( user_j == 0 )
-				user_state = user_start;
 			break;
 
 			case UsrrpOp:
 			// parameter result in an atomic operation sequence
-			CPPAD_ASSERT_UNKNOWN( user_state == user_ret );
-			CPPAD_ASSERT_UNKNOWN( 0 < user_i && user_i <= user_m );
-			CPPAD_ASSERT_UNKNOWN( NumArg(op) == 1 );
 			CPPAD_ASSERT_UNKNOWN( size_t(arg[0]) < num_par );
-			--user_i;
-			if( user_i == 0 )
-				user_state = user_arg;
+			play->reverse_user(op, user_state,
+				user_old, user_m, user_n, user_i, user_j
+			);
 			break;
 
 			case UsrrvOp:
 			// variable result in an atomic operation sequence
-			CPPAD_ASSERT_UNKNOWN( user_state == user_ret );
-			CPPAD_ASSERT_UNKNOWN( 0 < user_i && user_i <= user_m );
-			--user_i;
+			play->reverse_user(op, user_state,
+				user_old, user_m, user_n, user_i, user_j
+			);
 			if( RevJac[i_var] )
 			{
 				user_s[user_i] = true;
 			}
-			rev_hes_sparse.begin(i_var);
-			j = rev_hes_sparse.next_element();
-			while( j < user_q )
-			{	if( user_bool )
-					bool_u[user_i * user_q + j] = true;
-				else
-					set_u[user_i].insert(j);
-				j = rev_hes_sparse.next_element();
+			{	typename Vector_set::const_iterator itr(rev_hes_sparse, i_var);
+				j = *itr;
+				while( j < user_q )
+				{	if( user_pack )
+						pack_u[user_i * user_q + j] = true;
+					if( user_bool )
+						bool_u[user_i * user_q + j] = true;
+					if( user_set )
+						set_u[user_i].insert(j);
+					j = *(++itr);
+				}
 			}
-			if( user_i == 0 )
-				user_state = user_arg;
+			break;
+			// -------------------------------------------------
+
+			case ZmulpvOp:
+			CPPAD_ASSERT_NARG_NRES(op, 2, 1)
+			reverse_sparse_hessian_linear_unary_op(
+			i_var, arg[1], RevJac, for_jac_sparse, rev_hes_sparse
+			);
+			break;
+			// -------------------------------------------------
+
+			case ZmulvpOp:
+			CPPAD_ASSERT_NARG_NRES(op, 2, 1)
+			reverse_sparse_hessian_linear_unary_op(
+			i_var, arg[0], RevJac, for_jac_sparse, rev_hes_sparse
+			);
+			break;
+			// -------------------------------------------------
+
+			case ZmulvvOp:
+			CPPAD_ASSERT_NARG_NRES(op, 2, 1)
+			reverse_sparse_hessian_mul_op(
+			i_var, arg, RevJac, for_jac_sparse, rev_hes_sparse
+			);
 			break;
 
 			// -------------------------------------------------
@@ -775,42 +899,48 @@ void RevHesSweep(
 		{	zf_value[j] = false;
 			zh_value[j] = false;
 		}
-		for_jac_sparse.begin(i_var);;
-		j = for_jac_sparse.next_element();;
+		typename Vector_set::const_iterator itr_jac(for_jac_sparse, i_var);
+		j = *itr_jac;
 		while( j < limit )
 		{	zf_value[j] = true;
-			j = for_jac_sparse.next_element();
+			j = *(++itr_jac);
 		}
-		rev_hes_sparse.begin(i_var);;
-		j = rev_hes_sparse.next_element();;
+		typename Vector_set::const_iterator itr_hes(rev_hes_sparse, i_var);
+		j = *itr_hes;
 		while( j < limit )
 		{	zh_value[j] = true;
-			j = rev_hes_sparse.next_element();
+			j = *(++itr_hes);
 		}
-		// should also print RevJac[i_var], but printOp does not
-		// yet allow for this.
 		printOp(
-			std::cout, 
+			std::cout,
 			play,
 			i_op,
 			i_var,
-			op, 
-			arg,
-			1, 
-			&zf_value, 
-			1, 
+			op,
+			arg
+		);
+		// should also print RevJac[i_var], but printOpResult does not
+		// yet allow for this
+		if( NumRes(op) > 0 && op != BeginOp ) printOpResult(
+			std::cout,
+			1,
+			&zf_value,
+			1,
 			&zh_value
 		);
-# endif
+		std::cout << std::endl;
 	}
+	std::cout << std::endl;
+# else
+	}
+# endif
 	// values corresponding to BeginOp
 	CPPAD_ASSERT_UNKNOWN( i_op == 0 );
 	CPPAD_ASSERT_UNKNOWN( i_var == 0 );
 
 	return;
 }
-/*! \} */
-} // END_CPPAD_NAMESPACE
+} } // END_CPPAD_LOCAL_NAMESPACE
 
 // preprocessor symbols that are local to this file
 # undef CPPAD_REV_HES_SWEEP_TRACE
