@@ -1,12 +1,12 @@
-/* $Id: reverse_sweep.hpp 2991 2013-10-22 16:25:15Z bradbell $ */
-# ifndef CPPAD_REVERSE_SWEEP_INCLUDED
-# define CPPAD_REVERSE_SWEEP_INCLUDED
+// $Id: reverse_sweep.hpp 3853 2016-12-14 14:40:11Z bradbell $
+# ifndef CPPAD_LOCAL_REVERSE_SWEEP_HPP
+# define CPPAD_LOCAL_REVERSE_SWEEP_HPP
 
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-13 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
-the terms of the 
+the terms of the
                     Eclipse Public License Version 1.0.
 
 A copy of this license is included in the COPYING file of this distribution.
@@ -14,10 +14,8 @@ Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 -------------------------------------------------------------------------- */
 
 
-namespace CppAD { // BEGIN_CPPAD_NAMESPACE
+namespace CppAD { namespace local { // BEGIN_CPPAD_LOCAL_NAMESPACE
 /*!
-\defgroup reverse_sweep_hpp reverse_sweep.hpp
-\{
 \file reverse_sweep.hpp
 Compute derivatives of arbitrary order Taylor coefficients.
 */
@@ -33,7 +31,7 @@ otherwise, it respolves to
 \code
 	user_ok = user_atom->reverse
 \endcode
-This maco is undefined at the end of this file to facillitate is 
+This maco is undefined at the end of this file to facillitate is
 use with a different definition in other files.
 */
 # ifdef NDEBUG
@@ -44,7 +42,7 @@ use with a different definition in other files.
 
 /*!
 \def CPPAD_REVERSE_SWEEP_TRACE
-This value is either zero or one. 
+This value is either zero or one.
 Zero is the normal operational value.
 If it is one, a trace of every reverse_sweep computation is printed.
 */
@@ -55,11 +53,11 @@ Compute derivative of arbitrary order forward mode Taylor coefficients.
 
 \tparam Base
 base type for the operator; i.e., this operation sequence was recorded
-using AD< \a Base > and computations by this routine are done using type 
+using AD< \a Base > and computations by this routine are done using type
 \a Base.
 
 \param d
-is the highest order Taylor coefficients that 
+is the highest order Taylor coefficients that
 we are computing the derivative of.
 
 \param n
@@ -68,42 +66,53 @@ is the number of independent variables on the tape.
 \param numvar
 is the total number of variables on the tape.
 This is also equal to the number of rows in the matrix \a Taylor; i.e.,
-Rec->num_rec_var().
+play->num_var_rec().
 
-\param Rec
-2DO: change this name from Rec to play (becuase it is a player 
-and not a recorder).
-The information stored in \a Rec
+\param play
+The information stored in \a play
 is a recording of the operations corresponding to the function
 \f[
 	F : {\bf R}^n \rightarrow {\bf R}^m
 \f]
 where \f$ n \f$ is the number of independent variables and
 \f$ m \f$ is the number of dependent variables.
-We define the function 
-\f$ G : {\bf R}^{n \times d} \rightarrow {\bf R} \f$ by
+We define \f$ u^{(k)} \f$ as the value of <code>x_k</code> in the previous call
+of the form
+<code>
+	f.Forward(k, x_k)
+</code>
+We define
+\f$ X : {\bf R}^{n \times d} \rightarrow {\bf R}^n \f$ by
 \f[
-G( u ) = \frac{1}{d !} \frac{ \partial^d }{ \partial t^d } 
-\left[ 
-	\sum_{i=1}^m w_i  F_i ( u^{(0)} + u^{(1)} t + \cdots + u^{(d)} t^d )
-\right]_{t=0}
+	X(t, u) =  u^{(0)} + u^{(1)} t + \cdots + u^{(d)} t^d
 \f]
-Note that the scale factor  1 / a d  converts 
-the \a d-th partial derivative to the \a d-th order Taylor coefficient.
-This routine computes the derivative of \f$ G(u) \f$
+We define
+\f$ Y : {\bf R}^{n \times d} \rightarrow {\bf R}^m \f$ by
+\f[
+	Y(t, u) =  F[ X(t, u) ]
+\f]
+We define the function
+\f$ W : {\bf R}^{n \times d} \rightarrow {\bf R} \f$ by
+\f[
+W(u)
+=
+\sum_{k=0}^{d} ( w^{(k)} )^{\rm T}
+	\frac{1}{k !} \frac{\partial^k}{\partial t^k} Y(0, u)
+\f]
+(The matrix \f$ w \in {\bf R}^m \f$,
+is defined below under the heading Partial.)
+Note that the scale factor  1 / k  converts
+the k-th partial derivative to the k-th order Taylor coefficient.
+This routine computes the derivative of \f$ W(u) \f$
 with respect to all the Taylor coefficients
 \f$ u^{(k)} \f$ for \f$ k = 0 , ... , d \f$.
-The vector \f$ w \in {\bf R}^m \f$, and
-value of \f$ u \in {\bf R}^{n \times d} \f$
-at which the derivative is computed,
-are defined below.
 \n
 \n
-The object \a Rec is effectly constant.
+The object \a play is effectly constant.
 There is an exception to this,
 while palying back the tape
-the object \a Rec holds information about the current location
-with in the tape and this changes during palyback. 
+the object \a play holds information about the current location
+with in the tape and this changes during palyback.
 
 \param J
 Is the number of columns in the coefficient matrix \a Taylor.
@@ -112,7 +121,7 @@ This must be greater than or equal \a d + 1.
 \param Taylor
 For i = 1 , ... , \a numvar, and for k = 0 , ... , \a d,
 \a Taylor [ i * J + k ]
-is the k-th order Taylor coefficient corresponding to 
+is the k-th order Taylor coefficient corresponding to
 variable with index i on the tape.
 The value \f$ u \in {\bf R}^{n \times d} \f$,
 at which the derivative is computed,
@@ -127,34 +136,42 @@ It must be greater than or equal \a d + 1.
 \param Partial
 \b Input:
 The last \f$ m \f$ rows of \a Partial are inputs.
-The vector \f$ v \f$, used to define \f$ G(u) \f$,
-is specified by these rows. 
-For i = 0 , ... , m - 1, \a Partial [ ( \a numvar - m + i ) * K + d ] = v_i.
-For i = 0 , ... , m - 1 and for k = 0 , ... , d - 1, 
-\a Partial [ ( \a numvar - m + i ) * K + k ] = 0.
+The matrix \f$ w \f$, used to define \f$ W(u) \f$,
+is specified by these rows.
+For i = 0 , ... , m - 1,
+for k = 0 , ... , d,
+<code>Partial [ (numvar - m + i ) * K + k ] = w[i,k]</code>.
 \n
 \n
 \b Temporary:
-For i = n+1 , ... , \a numvar - 1 and for k = 0 , ... , d, 
+For i = n+1 , ... , \a numvar - 1 and for k = 0 , ... , d,
 the value of \a Partial [ i * K + k ] is used for temporary work space
-and its output value is not defined. 
+and its output value is not defined.
 \n
 \n
 \b Output:
-For j = 1 , ... , n and for k = 0 , ... , d, 
-\a Partial [ j * K + k ] 
-is the partial derivative of \f$ G( u ) \f$ with 
+For j = 1 , ... , n and for k = 0 , ... , d,
+\a Partial [ j * K + k ]
+is the partial derivative of \f$ W( u ) \f$ with
 respect to \f$ u_j^{(k)} \f$.
 
 \param cskip_op
-Is a vector with size Rec->num_rec_op().
+Is a vector with size play->num_op_rec().
 If cskip_op[i] is true, the operator index i in the recording
 does not affect any of the dependent variable (given the value
 of the independent variables).
+Note that all the operators in an atomic function call are skipped as a block,
+so only the last UserOp fore each call needs to have cskip_op[i] true.
+
+\param var_by_load_op
+is a vector with size play->num_load_op_rec().
+Is the variable index corresponding to each load instruction.
+In the case where the index is zero,
+the instruction corresponds to a parameter (not variable).
 
 \par Assumptions
 The first operator on the tape is a BeginOp,
-and the next \a n operators are InvOp operations for the 
+and the next \a n operators are InvOp operations for the
 corresponding independent variables.
 */
 template <class Base>
@@ -162,12 +179,13 @@ void ReverseSweep(
 	size_t                      d,
 	size_t                      n,
 	size_t                      numvar,
-	player<Base>*               Rec,
+	local::player<Base>*               play,
 	size_t                      J,
 	const Base*                 Taylor,
 	size_t                      K,
 	Base*                       Partial,
-	const CppAD::vector<bool>&  cskip_op
+	bool*                       cskip_op,
+	const pod_vector<addr_t>&   var_by_load_op
 )
 {
 	OpCode           op;
@@ -177,16 +195,16 @@ void ReverseSweep(
 	const addr_t*   arg = CPPAD_NULL;
 
 	// check numvar argument
-	CPPAD_ASSERT_UNKNOWN( Rec->num_rec_var() == numvar );
+	CPPAD_ASSERT_UNKNOWN( play->num_var_rec() == numvar );
 	CPPAD_ASSERT_UNKNOWN( numvar > 0 );
 
 	// length of the parameter vector (used by CppAD assert macros)
-	const size_t num_par = Rec->num_rec_par();
+	const size_t num_par = play->num_par_rec();
 
 	// pointer to the beginning of the parameter vector
 	const Base* parameter = CPPAD_NULL;
 	if( num_par > 0 )
-		parameter = Rec->GetPar();
+		parameter = play->GetPar();
 
 	// work space used by UserOp.
 	const size_t user_k  = d;    // highest order we are differentiating
@@ -196,66 +214,102 @@ void ReverseSweep(
 	vector<Base> user_ty;        // result vector Taylor coefficients
 	vector<Base> user_px;        // partials w.r.t argument vector
 	vector<Base> user_py;        // partials w.r.t. result vector
-	size_t user_index = 0;       // indentifier for this atomic operation
-	size_t user_id    = 0;       // user identifier for this call to operator
-	size_t user_i     = 0;       // index in result vector
-	size_t user_j     = 0;       // index in argument vector
-	size_t user_m     = 0;       // size of result vector
-	size_t user_n     = 0;       // size of arugment vector
 	//
 	atomic_base<Base>* user_atom = CPPAD_NULL; // user's atomic op calculator
 # ifndef NDEBUG
 	bool               user_ok   = false;      // atomic op return value
 # endif
 	//
-	// next expected operator in a UserOp sequence
-	enum { user_start, user_arg, user_ret, user_end } user_state = user_end;
+	// information defined by forward_user
+	size_t user_old=0, user_m=0, user_n=0, user_i=0, user_j=0;
+	enum_user_state user_state = end_user; // proper initialization
 
 	// temporary indices
 	size_t j, ell;
 
 	// Initialize
-	Rec->start_reverse(op, arg, i_op, i_var);
+	play->reverse_start(op, arg, i_op, i_var);
 	CPPAD_ASSERT_UNKNOWN( op == EndOp );
 # if CPPAD_REVERSE_SWEEP_TRACE
 	std::cout << std::endl;
 # endif
 	bool more_operators = true;
 	while(more_operators)
-	{	// next op
-		Rec->next_reverse(op, arg, i_op, i_var);
+	{	bool flag; // temporary for use in switch cases
+		//
+		// next op
+		play->reverse_next(op, arg, i_op, i_var);
 		CPPAD_ASSERT_UNKNOWN((i_op >  n) | (op == InvOp) | (op == BeginOp));
 		CPPAD_ASSERT_UNKNOWN((i_op <= n) | (op != InvOp) | (op != BeginOp));
+		CPPAD_ASSERT_UNKNOWN( i_op < play->num_op_rec() );
 
 		// check if we are skipping this operation
 		while( cskip_op[i_op] )
-		{	if( op == CSumOp )
-			{	// CSumOp has a variable number of arguments
-				Rec->reverse_csum(op, arg, i_op, i_var);
+		{	switch(op)
+			{	case CSumOp:
+				// CSumOp has a variable number of arguments
+				play->reverse_csum(op, arg, i_op, i_var);
+				break;
+
+				case CSkipOp:
+				// CSkip has a variable number of arguments
+				play->reverse_cskip(op, arg, i_op, i_var);
+				break;
+
+				case UserOp:
+				{	// skip all operations in this user atomic call
+					CPPAD_ASSERT_UNKNOWN( user_state == end_user );
+					play->reverse_user(op, user_state,
+						user_old, user_m, user_n, user_i, user_j
+					);
+					size_t n_skip = user_m + user_n + 1;
+					for(size_t i = 0; i < n_skip; i++)
+					{	play->reverse_next(op, arg, i_op, i_var);
+						play->reverse_user(op, user_state,
+							user_old, user_m, user_n, user_i, user_j
+						);
+					}
+					CPPAD_ASSERT_UNKNOWN( user_state == end_user );
+				}
+				break;
+
+				default:
+				break;
 			}
-			Rec->next_reverse(op, arg, i_op, i_var);
+			play->reverse_next(op, arg, i_op, i_var);
+			CPPAD_ASSERT_UNKNOWN( i_op < play->num_op_rec() );
 		}
 
 		// rest of informaiton depends on the case
 # if CPPAD_REVERSE_SWEEP_TRACE
+		if( op == CSumOp )
+		{	// CSumOp has a variable number of arguments
+			play->reverse_csum(op, arg, i_op, i_var);
+		}
+		if( op == CSkipOp )
+		{	// CSkip has a variable number of arguments
+			play->reverse_cskip(op, arg, i_op, i_var);
+		}
 		size_t       i_tmp  = i_var;
 		const Base*  Z_tmp  = Taylor + i_var * J;
 		const Base*  pZ_tmp = Partial + i_var * K;
-
 		printOp(
-			std::cout, 
-			Rec,
+			std::cout,
+			play,
 			i_op,
 			i_tmp,
-			op, 
-			arg,
-			d + 1, 
-			Z_tmp, 
-			d + 1, 
-			pZ_tmp 
+			op,
+			arg
 		);
+		if( NumRes(op) > 0 && op != BeginOp ) printOpResult(
+			std::cout,
+			d + 1,
+			Z_tmp,
+			d + 1,
+			pZ_tmp
+		);
+		std::cout << std::endl;
 # endif
-
 		switch( op )
 		{
 
@@ -266,6 +320,26 @@ void ReverseSweep(
 			break;
 			// --------------------------------------------------
 
+			case AcosOp:
+			// sqrt(1 - x * x), acos(x)
+			CPPAD_ASSERT_UNKNOWN( i_var < numvar );
+			reverse_acos_op(
+				d, i_var, arg[0], J, Taylor, K, Partial
+			);
+			break;
+			// --------------------------------------------------
+
+# if CPPAD_USE_CPLUSPLUS_2011
+			case AcoshOp:
+			// sqrt(x * x - 1), acosh(x)
+			CPPAD_ASSERT_UNKNOWN( i_var < numvar );
+			reverse_acosh_op(
+				d, i_var, arg[0], J, Taylor, K, Partial
+			);
+			break;
+# endif
+			// --------------------------------------------------
+
 			case AddvvOp:
 			reverse_addvv_op(
 				d, i_var, arg, parameter, J, Taylor, K, Partial
@@ -281,17 +355,8 @@ void ReverseSweep(
 			break;
 			// --------------------------------------------------
 
-			case AcosOp:
-                        // sqrt(1 - x * x), acos(x)
-			CPPAD_ASSERT_UNKNOWN( i_var < numvar );
-			reverse_acos_op(
-				d, i_var, arg[0], J, Taylor, K, Partial
-			);
-			break;
-			// --------------------------------------------------
-
 			case AsinOp:
-                        // sqrt(1 - x * x), asin(x)
+			// sqrt(1 - x * x), asin(x)
 			CPPAD_ASSERT_UNKNOWN( i_var < numvar );
 			reverse_asin_op(
 				d, i_var, arg[0], J, Taylor, K, Partial
@@ -299,8 +364,19 @@ void ReverseSweep(
 			break;
 			// --------------------------------------------------
 
+# if CPPAD_USE_CPLUSPLUS_2011
+			case AsinhOp:
+			// sqrt(1 + x * x), asinh(x)
+			CPPAD_ASSERT_UNKNOWN( i_var < numvar );
+			reverse_asinh_op(
+				d, i_var, arg[0], J, Taylor, K, Partial
+			);
+			break;
+# endif
+			// --------------------------------------------------
+
 			case AtanOp:
-                        // 1 + x * x, atan(x)
+			// 1 + x * x, atan(x)
 			CPPAD_ASSERT_UNKNOWN( i_var < numvar );
 			reverse_atan_op(
 				d, i_var, arg[0], J, Taylor, K, Partial
@@ -308,6 +384,17 @@ void ReverseSweep(
 			break;
 			// -------------------------------------------------
 
+# if CPPAD_USE_CPLUSPLUS_2011
+			case AtanhOp:
+			// 1 - x * x, atanh(x)
+			CPPAD_ASSERT_UNKNOWN( i_var < numvar );
+			reverse_atanh_op(
+				d, i_var, arg[0], J, Taylor, K, Partial
+			);
+			break;
+# endif
+			// -------------------------------------------------
+
 			case BeginOp:
 			CPPAD_ASSERT_NARG_NRES(op, 1, 1);
 			more_operators = false;
@@ -316,17 +403,21 @@ void ReverseSweep(
 
 			case CSkipOp:
 			// CSkipOp has a variable number of arguments and
-			// next_forward thinks it one has one argument.
-			// we must inform next_forward of this special case.
-			Rec->reverse_cskip(op, arg, i_op, i_var);
+			// forward_next thinks it one has one argument.
+			// we must inform reverse_next of this special case.
+# if ! CPPAD_REVERSE_SWEEP_TRACE
+			play->reverse_cskip(op, arg, i_op, i_var);
+# endif
 			break;
 			// -------------------------------------------------
 
 			case CSumOp:
 			// CSumOp has a variable number of arguments and
-			// next_reverse thinks it one has one argument.
-			// We must inform next_reverse of this special case.
-			Rec->reverse_csum(op, arg, i_op, i_var);
+			// reverse_next thinks it one has one argument.
+			// We must inform reverse_next of this special case.
+# if ! CPPAD_REVERSE_SWEEP_TRACE
+			play->reverse_csum(op, arg, i_op, i_var);
+# endif
 			reverse_csum_op(
 				d, i_var, arg, K, Partial
 			);
@@ -336,23 +427,19 @@ void ReverseSweep(
 
 			case CExpOp:
 			reverse_cond_op(
-				d, 
-				i_var, 
-				arg, 
-				num_par, 
-				parameter, 
-				J, 
+				d,
+				i_var,
+				arg,
+				num_par,
+				parameter,
+				J,
 				Taylor,
-				K, 
+				K,
 				Partial
 			);
 			break;
 			// --------------------------------------------------
 
-			case ComOp:
-			break;
-			// --------------------------------------------------
-
 			case CosOp:
 			CPPAD_ASSERT_UNKNOWN( i_var < numvar );
 			reverse_cos_op(
@@ -371,7 +458,7 @@ void ReverseSweep(
 
 			case DisOp:
 			// Derivative of discrete operation is zero so no
-			// contribution passes through this operation. 
+			// contribution passes through this operation.
 			break;
 			// --------------------------------------------------
 
@@ -398,30 +485,62 @@ void ReverseSweep(
 			break;
 			// --------------------------------------------------
 
+# if CPPAD_USE_CPLUSPLUS_2011
+			case ErfOp:
+			reverse_erf_op(
+				d, i_var, arg, parameter, J, Taylor, K, Partial
+			);
+			break;
+# endif
+			// --------------------------------------------------
+
 			case ExpOp:
 			reverse_exp_op(
 				d, i_var, arg[0], J, Taylor, K, Partial
 			);
 			break;
 			// --------------------------------------------------
+
+# if CPPAD_USE_CPLUSPLUS_2011
+			case Expm1Op:
+			reverse_expm1_op(
+				d, i_var, arg[0], J, Taylor, K, Partial
+			);
+			break;
+# endif
+			// --------------------------------------------------
+
+			case InvOp:
+			break;
+			// --------------------------------------------------
+
 			case LdpOp:
 			reverse_load_op(
-				op, d, i_var, arg, J, Taylor, K, Partial
+			op, d, i_var, arg, J, Taylor, K, Partial, var_by_load_op.data()
 			);
 			break;
 			// -------------------------------------------------
 
 			case LdvOp:
 			reverse_load_op(
-				op, d, i_var, arg, J, Taylor, K, Partial
+			op, d, i_var, arg, J, Taylor, K, Partial, var_by_load_op.data()
 			);
 			break;
-			// -------------------------------------------------
-
-			case InvOp:
-			break;
 			// --------------------------------------------------
 
+			case EqpvOp:
+			case EqvvOp:
+			case LtpvOp:
+			case LtvpOp:
+			case LtvvOp:
+			case LepvOp:
+			case LevpOp:
+			case LevvOp:
+			case NepvOp:
+			case NevvOp:
+			break;
+			// -------------------------------------------------
+
 			case LogOp:
 			reverse_log_op(
 				d, i_var, arg[0], J, Taylor, K, Partial
@@ -429,11 +548,13 @@ void ReverseSweep(
 			break;
 			// --------------------------------------------------
 
-			case MulvvOp:
-			reverse_mulvv_op(
-				d, i_var, arg, parameter, J, Taylor, K, Partial
+# if CPPAD_USE_CPLUSPLUS_2011
+			case Log1pOp:
+			reverse_log1p_op(
+				d, i_var, arg[0], J, Taylor, K, Partial
 			);
 			break;
+# endif
 			// --------------------------------------------------
 
 			case MulpvOp:
@@ -444,6 +565,13 @@ void ReverseSweep(
 			break;
 			// --------------------------------------------------
 
+			case MulvvOp:
+			reverse_mulvv_op(
+				d, i_var, arg, parameter, J, Taylor, K, Partial
+			);
+			break;
+			// --------------------------------------------------
+
 			case ParOp:
 			break;
 			// --------------------------------------------------
@@ -564,24 +692,12 @@ void ReverseSweep(
 
 			case UserOp:
 			// start or end an atomic operation sequence
-			CPPAD_ASSERT_UNKNOWN( NumRes( UserOp ) == 0 );
-			CPPAD_ASSERT_UNKNOWN( NumArg( UserOp ) == 4 );
-			if( user_state == user_end )
-			{	user_index = arg[0];
-				user_id    = arg[1];
-				user_n     = arg[2];
-				user_m     = arg[3];
-				user_atom  = atomic_base<Base>::class_object(user_index);
-# ifndef NDEBUG
-				if( user_atom == CPPAD_NULL )
-				{	std::string msg = 
-						atomic_base<Base>::class_name(user_index)
-						+ ": atomic_base function has been deleted";
-					CPPAD_ASSERT_KNOWN(false, msg.c_str() );
-				}
-# endif
-				if(user_ix.size() != user_n)
-					user_ix.resize(user_n);
+			flag = user_state == end_user;
+			user_atom = play->reverse_user(op, user_state,
+				user_old, user_m, user_n, user_i, user_j
+			);
+			if( flag )
+			{	user_ix.resize(user_n);
 				if(user_tx.size() != user_n * user_k1)
 				{	user_tx.resize(user_n * user_k1);
 					user_px.resize(user_n * user_k1);
@@ -590,26 +706,17 @@ void ReverseSweep(
 				{	user_ty.resize(user_m * user_k1);
 					user_py.resize(user_m * user_k1);
 				}
-				user_j     = user_n;
-				user_i     = user_m;
-				user_state = user_ret;
 			}
 			else
-			{	CPPAD_ASSERT_UNKNOWN( user_state == user_start );
-				CPPAD_ASSERT_UNKNOWN( user_index == size_t(arg[0]) );
-				CPPAD_ASSERT_UNKNOWN( user_id    == size_t(arg[1]) );
-				CPPAD_ASSERT_UNKNOWN( user_n     == size_t(arg[2]) );
-				CPPAD_ASSERT_UNKNOWN( user_m     == size_t(arg[3]) );
-
-				// call users function for this operation
-				user_atom->set_id(user_id);
+			{	// call users function for this operation
+				user_atom->set_old(user_old);
 				CPPAD_ATOMIC_CALL(
 					user_k, user_tx, user_ty, user_px, user_py
 				);
 # ifndef NDEBUG
 				if( ! user_ok )
-				{	std::string msg = 
-						atomic_base<Base>::class_name(user_index)
+				{	std::string msg =
+						user_atom->afun_name()
 						+ ": atomic_base.reverse: returned false";
 					CPPAD_ASSERT_KNOWN(false, msg.c_str() );
 				}
@@ -619,73 +726,83 @@ void ReverseSweep(
 						Partial[user_ix[j] * K + ell] +=
 							user_px[j * user_k1 + ell];
 				}
-				user_state = user_end;
 			}
 			break;
 
 			case UsrapOp:
 			// parameter argument in an atomic operation sequence
-			CPPAD_ASSERT_UNKNOWN( user_state == user_arg );
-			CPPAD_ASSERT_UNKNOWN( 0 < user_j && user_j <= user_n );
-			CPPAD_ASSERT_UNKNOWN( NumArg(op) == 1 );
 			CPPAD_ASSERT_UNKNOWN( size_t(arg[0]) < num_par );
-			--user_j;
+			play->reverse_user(op, user_state,
+				user_old, user_m, user_n, user_i, user_j
+			);
 			user_ix[user_j] = 0;
 			user_tx[user_j * user_k1 + 0] = parameter[ arg[0]];
 			for(ell = 1; ell < user_k1; ell++)
 				user_tx[user_j * user_k1 + ell] = Base(0.);
-			
-			if( user_j == 0 )
-				user_state = user_start;
 			break;
 
 			case UsravOp:
 			// variable argument in an atomic operation sequence
-			CPPAD_ASSERT_UNKNOWN( user_state == user_arg );
-			CPPAD_ASSERT_UNKNOWN( 0 < user_j && user_j <= user_n );
-			CPPAD_ASSERT_UNKNOWN( NumArg(op) == 1 );
 			CPPAD_ASSERT_UNKNOWN( size_t(arg[0]) <= i_var );
 			CPPAD_ASSERT_UNKNOWN( 0 < arg[0] );
-			--user_j;
+			play->reverse_user(op, user_state,
+				user_old, user_m, user_n, user_i, user_j
+			);
 			user_ix[user_j] = arg[0];
 			for(ell = 0; ell < user_k1; ell++)
 				user_tx[user_j*user_k1 + ell] = Taylor[ arg[0] * J + ell];
-			if( user_j == 0 )
-				user_state = user_start;
 			break;
 
 			case UsrrpOp:
 			// parameter result in an atomic operation sequence
-			CPPAD_ASSERT_UNKNOWN( user_state == user_ret );
-			CPPAD_ASSERT_UNKNOWN( 0 < user_i && user_i <= user_m );
-			CPPAD_ASSERT_UNKNOWN( NumArg(op) == 1 );
 			CPPAD_ASSERT_UNKNOWN( size_t(arg[0]) < num_par );
-			--user_i;
+			play->reverse_user(op, user_state,
+				user_old, user_m, user_n, user_i, user_j
+			);
 			for(ell = 0; ell < user_k1; ell++)
 			{	user_py[user_i * user_k1 + ell] = Base(0.);
 				user_ty[user_i * user_k1 + ell] = Base(0.);
 			}
 			user_ty[user_i * user_k1 + 0] = parameter[ arg[0] ];
-			if( user_i == 0 )
-				user_state = user_arg;
 			break;
 
 			case UsrrvOp:
 			// variable result in an atomic operation sequence
-			CPPAD_ASSERT_UNKNOWN( user_state == user_ret );
-			CPPAD_ASSERT_UNKNOWN( 0 < user_i && user_i <= user_m );
-			--user_i;
+			play->reverse_user(op, user_state,
+				user_old, user_m, user_n, user_i, user_j
+			);
 			for(ell = 0; ell < user_k1; ell++)
 			{	user_py[user_i * user_k1 + ell] =
 						Partial[i_var * K + ell];
 				user_ty[user_i * user_k1 + ell] =
 						Taylor[i_var * J + ell];
 			}
-			if( user_i == 0 )
-				user_state = user_arg;
 			break;
 			// ------------------------------------------------------------
 
+			case ZmulpvOp:
+			CPPAD_ASSERT_UNKNOWN( size_t(arg[0]) < num_par );
+			reverse_zmulpv_op(
+				d, i_var, arg, parameter, J, Taylor, K, Partial
+			);
+			break;
+			// --------------------------------------------------
+
+			case ZmulvpOp:
+			CPPAD_ASSERT_UNKNOWN( size_t(arg[1]) < num_par );
+			reverse_zmulvp_op(
+				d, i_var, arg, parameter, J, Taylor, K, Partial
+			);
+			break;
+			// --------------------------------------------------
+
+			case ZmulvvOp:
+			reverse_zmulvv_op(
+				d, i_var, arg, parameter, J, Taylor, K, Partial
+			);
+			break;
+			// --------------------------------------------------
+
 			default:
 			CPPAD_ASSERT_UNKNOWN(false);
 		}
@@ -698,8 +815,7 @@ void ReverseSweep(
 	CPPAD_ASSERT_UNKNOWN( i_var == 0 );
 }
 
-/*! \} */
-} // END_CPPAD_NAMESPACE
+} } // END_CPPAD_LOCAL_NAMESPACE
 
 // preprocessor symbols that are local to this file
 # undef CPPAD_REVERSE_SWEEP_TRACE
