@@ -1,11 +1,11 @@
-/* $Id: harmonic.hpp 2506 2012-10-24 19:36:49Z bradbell $ */
-# ifndef CPPAD_HARMONIC_INCLUDED
-# define CPPAD_HARMONIC_INCLUDED
+// $Id: harmonic.hpp 3804 2016-03-20 15:08:46Z bradbell $
+# ifndef CPPAD_MULTI_THREAD_HARMONIC_HPP
+# define CPPAD_MULTI_THREAD_HARMONIC_HPP
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-11 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
-the terms of the 
+the terms of the
                     Eclipse Public License Version 1.0.
 
 A copy of this license is included in the COPYING file of this distribution.
