@@ -1,9 +1,9 @@
-/* $Id: ode.cpp 3311 2014-05-28 16:21:08Z bradbell $ */
+// $Id: ode.cpp 3853 2016-12-14 14:40:11Z bradbell $
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-14 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
-the terms of the 
+the terms of the
                     Eclipse Public License Version 1.0.
 
 A copy of this license is included in the COPYING file of this distribution.
@@ -29,27 +29,23 @@ $spell
 $$
 
 $section CppAD Speed: Gradient of Ode Solution$$
+$mindex link_ode speed$$
 
-$index link_ode, cppad$$
-$index cppad, link_ode$$
-$index speed, cppad$$
-$index cppad, speed$$
-$index ode, speed cppad$$
 
 $head Specifications$$
 See $cref link_ode$$.
 
 $head Implementation$$
 
-$codep */
+$srccode%cpp% */
 # include <cppad/cppad.hpp>
 # include <cppad/speed/ode_evaluate.hpp>
 # include <cppad/speed/uniform_01.hpp>
 # include <cassert>
 
-// Note that CppAD uses global_memory at the main program level
-extern bool
-	global_onetape, global_atomic, global_optimize;
+// Note that CppAD uses global_option["memory"] at the main program level
+# include <map>
+extern std::map<std::string, bool> global_option;
 
 bool link_ode(
 	size_t                     size       ,
@@ -59,9 +55,11 @@ bool link_ode(
 )
 {
 	// speed test global option values
-	if( global_atomic )
+	if( global_option["atomic"] )
 		return false;
 
+	// optimization options: no conditional skips or compare operators
+	std::string options="no_compare_op";
 	// --------------------------------------------------------------------
 	// setup
 	assert( x.size() == size );
@@ -78,8 +76,8 @@ bool link_ode(
 	CppAD::ADFun<double>  f;   // AD function
 
 	// -------------------------------------------------------------
-	if( ! global_onetape ) while(repeat--)
-	{ 	// choose next x value
+	if( ! global_option["onetape"] ) while(repeat--)
+	{	// choose next x value
 		uniform_01(n, x);
 		for(j = 0; j < n; j++)
 			X[j] = x[j];
@@ -93,13 +91,16 @@ bool link_ode(
 		// create function object f : X -> Y
 		f.Dependent(X, Y);
 
-		if( global_optimize )
-			f.optimize();
+		if( global_option["optimize"] )
+			f.optimize(options);
+
+		// skip comparison operators
+		f.compare_change_count(0);
 
 		jacobian = f.Jacobian(x);
 	}
 	else
-	{ 	// an x value
+	{	// an x value
 		uniform_01(n, x);
 		for(j = 0; j < n; j++)
 			X[j] = x[j];
@@ -113,8 +114,12 @@ bool link_ode(
 		// create function object f : X -> Y
 		f.Dependent(X, Y);
 
-		if( global_optimize )
-			f.optimize();
+		if( global_option["optimize"] )
+			f.optimize(options);
+
+		// skip comparison operators
+		f.compare_change_count(0);
+
 		while(repeat--)
 		{	// get next argument value
 			uniform_01(n, x);
@@ -125,6 +130,6 @@ bool link_ode(
 	}
 	return true;
 }
-/* $$
+/* %$$
 $end
 */
