@@ -1,9 +1,9 @@
-/* $Id: ode.cpp 3311 2014-05-28 16:21:08Z bradbell $ */
+// $Id: ode.cpp 3794 2016-02-29 20:42:44Z bradbell $
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-14 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
-the terms of the 
+the terms of the
                     Eclipse Public License Version 1.0.
 
 A copy of this license is included in the COPYING file of this distribution.
@@ -25,27 +25,23 @@ $spell
 $$
 
 $section Double Speed: Ode Solution$$
+$mindex link_ode speed$$
 
-$index link_ode, double$$
-$index double, link_ode$$
-$index speed, double$$
-$index double, speed$$
-$index ode, speed double$$
 
 $head Specifications$$
 See $cref link_ode$$.
 
 $head Implementation$$
 
-$codep */
+$srccode%cpp% */
 # include <cstring>
-# include <cppad/vector.hpp>
+# include <cppad/utility/vector.hpp>
 # include <cppad/speed/ode_evaluate.hpp>
 # include <cppad/speed/uniform_01.hpp>
 
-// Note that CppAD uses global_memory at the main program level
-extern bool
-	global_onetape, global_atomic, global_optimize;
+// Note that CppAD uses global_option["memory"] at the main program level
+# include <map>
+extern std::map<std::string, bool> global_option;
 
 bool link_ode(
 	size_t                     size       ,
@@ -54,7 +50,7 @@ bool link_ode(
 	CppAD::vector<double>      &jacobian
 )
 {
-	if(global_onetape||global_atomic||global_optimize)
+	if(global_option["onetape"]||global_option["atomic"]||global_option["optimize"])
 		return false;
 	// -------------------------------------------------------------
 	// setup
@@ -66,7 +62,7 @@ bool link_ode(
 	CppAD::vector<double> f(n);
 
 	while(repeat--)
-	{ 	// choose next x value
+	{	// choose next x value
 		uniform_01(n, x);
 
 		// evaluate function
@@ -78,6 +74,6 @@ bool link_ode(
 		jacobian[i] = f[i];
 	return true;
 }
-/* $$
+/* %$$
 $end
 */
