@@ -1,9 +1,9 @@
-/* $Id: det_by_minor.cpp 2506 2012-10-24 19:36:49Z bradbell $ */
+// $Id: det_by_minor.cpp 3788 2016-02-09 15:50:06Z bradbell $
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-12 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
-the terms of the 
+the terms of the
                     Eclipse Public License Version 1.0.
 
 A copy of this license is included in the COPYING file of this distribution.
@@ -17,12 +17,10 @@ $spell
 $$
 
 $section Determinant Using Expansion by Minors: Example and Test$$
-$index determinant, by minors$$
-$index example, determinant by minors$$
-$index test, determinant by minors$$
+$mindex minors$$
 
 $code
-$verbatim%speed/example/det_by_minor.cpp%0%// BEGIN C++%// END C++%1%$$
+$srcfile%speed/example/det_by_minor.cpp%0%// BEGIN C++%// END C++%1%$$
 $$
 
 $end
