@@ -1,9 +1,9 @@
-/* $Id: mat_mul.cpp 3311 2014-05-28 16:21:08Z bradbell $ */
+// $Id: mat_mul.cpp 3794 2016-02-29 20:42:44Z bradbell $
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-14 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
-the terms of the 
+the terms of the
                     Eclipse Public License Version 1.0.
 
 A copy of this license is included in the COPYING file of this distribution.
@@ -27,37 +27,33 @@ $spell
 $$
 
 $section Fadbad Speed: Matrix Multiplication$$
+$mindex link_mat_mul speed multiply$$
 
-$index link_mat_mul, fadbad$$
-$index fadbad, link_mat_mul$$
-$index speed, fadbad$$
-$index fadbad, speed$$
-$index matrix, speed fadbad$$
-$index multiply, speed fadbad$$
 
 $head Specifications$$
 See $cref link_mat_mul$$.
 
 $head Implementation$$
 
-$codep */
+$srccode%cpp% */
 # include <FADBAD++/badiff.h>
 # include <cppad/speed/mat_sum_sq.hpp>
 # include <cppad/speed/uniform_01.hpp>
-# include <cppad/vector.hpp>
+# include <cppad/utility/vector.hpp>
 
 // list of possible options
-extern bool global_memory, global_onetape, global_atomic, global_optimize;
+# include <map>
+extern std::map<std::string, bool> global_option;
 
 bool link_mat_mul(
-	size_t                           size     , 
-	size_t                           repeat   , 
+	size_t                           size     ,
+	size_t                           repeat   ,
 	CppAD::vector<double>&           x        ,
 	CppAD::vector<double>&           z        ,
 	CppAD::vector<double>&           dz       )
 {
 	// speed test global option values
-	if( global_memory || global_onetape || global_atomic || global_optimize )
+	if( global_option["memory"] || global_option["onetape"] || global_option["atomic"] || global_option["optimize"] )
 		return false;
 	// The correctness check for this test is failing, so abort (for now).
 	return false;
@@ -66,8 +62,8 @@ bool link_mat_mul(
 	// setup
 
 	// object for computing determinant
-	typedef fadbad::B<double>       ADScalar; 
-	typedef CppAD::vector<ADScalar> ADVector; 
+	typedef fadbad::B<double>       ADScalar;
+	typedef CppAD::vector<ADScalar> ADVector;
 
 	size_t j;                // temporary index
 	size_t m = 1;            // number of dependent variables
@@ -75,7 +71,7 @@ bool link_mat_mul(
 	ADVector   X(n);         // AD domain space vector
 	ADVector   Y(n);         // Store product matrix
 	ADVector   Z(m);         // AD range space vector
-	
+
 	// ------------------------------------------------------
 	while(repeat--)
 	{	// get the next matrix
@@ -101,6 +97,6 @@ bool link_mat_mul(
 	// ---------------------------------------------------------
 	return true;
 }
-/* $$
+/* %$$
 $end
 */
