@@ -1,9 +1,9 @@
-/* $Id: main.cpp 2859 2013-05-28 06:03:21Z bradbell $ */
+// $Id: main.cpp 3855 2016-12-19 00:30:54Z bradbell $
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-13 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
-the terms of the 
+the terms of the
                     Eclipse Public License Version 1.0.
 
 A copy of this license is included in the COPYING file of this distribution.
@@ -16,43 +16,47 @@ Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 # include <cstddef>
 # include <iostream>
 # include <iomanip>
-# include <cppad/vector.hpp>
+# include <map>
+# include <cppad/utility/vector.hpp>
 # include <cppad/speed/det_grad_33.hpp>
 # include <cppad/speed/det_33.hpp>
-# include <cppad/time_test.hpp>
+# include <cppad/utility/time_test.hpp>
 # include <cppad/speed/uniform_01.hpp>
-# include <cppad/poly.hpp>
-# include <cppad/track_new_del.hpp>
-# include <cppad/thread_alloc.hpp>
+# include <cppad/utility/poly.hpp>
+# include <cppad/utility/track_new_del.hpp>
+# include <cppad/utility/thread_alloc.hpp>
 
-# ifdef SPEED_ADOLC
+# ifdef CPPAD_ADOLC_SPEED
 # define AD_PACKAGE "adolc"
 # endif
-# ifdef SPEED_CPPAD
+# ifdef CPPAD_CPPAD_SPEED
 # define AD_PACKAGE "cppad"
 # endif
-# ifdef SPEED_DOUBLE
+# ifdef CPPAD_DOUBLE_SPEED
 # define AD_PACKAGE "double"
 # endif
-# ifdef SPEED_FADBAD
+# ifdef CPPAD_FADBAD_SPEED
 # define AD_PACKAGE "fadbad"
 # endif
-# ifdef SPEED_PROFILE
+# ifdef CPPAD_PROFILE_SPEED
 # define AD_PACKAGE "profile"
 # endif
-# ifdef SPEED_SACADO
+# ifdef CPPAD_SACADO_SPEED
 # define AD_PACKAGE "sacado"
 # endif
 
 /*
 $begin speed_main$$
 $spell
+	revsparsity
+	colpack
+	onetape
+	boolsparsity
 	optionlist
 	underbar
 	alloc
 	mat_mul
 	retaped
-	retape
 	bool
 	ddp
 	cppad
@@ -65,14 +69,12 @@ $spell
 	Jacobian
 $$
 
-$index cppad, speed test$$
-$index speed, test cppad$$
-$index test, cppad speed$$
 
-$section Speed Testing Main Program$$
+$section Running the Speed Test Program$$
+$mindex cppad uniform_01$$
 
 $head Syntax$$
-$codei%speed/%package%/%package% %test% %seed% %option_list%$$
+$codei%speed/%package%/speed_%package% %test% %seed% %option_list%$$
 
 $head Purpose$$
 A version of this program runs the correctness tests
@@ -82,11 +84,19 @@ $head package$$
 
 $subhead AD Package$$
 The command line argument
-$icode package$$ specifies one of the following AD packages:
+$icode package$$ specifies one of the AD package.
+The CppAD distribution comes with support for the following packages:
 $cref/adolc/speed_adolc/$$,
-$cref/cppad/speed_cppad/$$, 
-$cref/fadbad/speed_fadbad/$$, 
+$cref/cppad/speed_cppad/$$,
+$cref/fadbad/speed_fadbad/$$,
 $cref/sacado/speed_sacado/$$.
+You can extend this program to include other package.
+Such an extension need not include all the tests.
+For example,
+$cref link_sparse_hessian$$ just returns $code false$$ for the
+$cref/fadbad/fadbad_sparse_hessian.cpp/$$ and
+$cref/sacado/sacado_sparse_hessian.cpp/$$ packages.
+
 
 $subhead double$$
 The value
@@ -96,7 +106,7 @@ using double precision operations.
 This enables one to compare the speed of computing function
 values in $code double$$ to the speed of the derivative computations.
 (It is often useful to divide the speed of the derivative computation by
-the speed of the function evaluation in $code double$$.) 
+the speed of the function evaluation in $code double$$.)
 
 $subhead profile$$
 In the special case where $icode package$$ is $code profile$$,
@@ -115,6 +125,8 @@ $cref/ode/link_ode/$$,
 $cref/poly/link_poly/$$,
 $cref/sparse_hessian/link_sparse_hessian/$$,
 $cref/sparse_jacobian/link_sparse_jacobian/$$.
+You can experiment with changing the implementation of a
+particular test for a particular package.
 
 $subhead correct$$
 If $icode test$$ is equal to $code correct$$,
@@ -125,7 +137,6 @@ If $icode test$$ is equal to $code speed$$,
 all of the speed tests are run.
 
 $head seed$$
-$index uniform_01$$
 The command line argument $icode seed$$ is a positive integer.
 The random number simulator $cref uniform_01$$ is initialized with
 the call
@@ -134,85 +145,126 @@ $codei%
 %$$
 before any of the testing routines (listed above) are called.
 
-$head option_list$$
-This is a list of options that follow $icode seed$$ value.
+$head Global Options$$
+This global variable has prototype
+$srccode%cpp%
+	extern std::map<std::string, bool> global_option;
+%$$
+This is true for each option that follows $icode seed$$.
 The order of the options does not matter and the list can be empty.
-Each option in the list, must be separate 
-command line argument to the main program. 
+Each option, must be separate
+command line argument to the main program.
+The documentation below specifics how CppAD uses these options,
+see the examples in $cref speed_adolc$$ for how another package might
+uses these options.
 
-$subhead retape$$
-If the option $code retape$$ is present, the symbol
+$subhead onetape$$
+If the option $code onetape$$ is present,
 $codep
-	extern bool global_retape
+	global_option["onetape"]
 $$
 is true and otherwise it is false.
 If this external symbol is true,
-every test must retape the 
-$cref/operation sequence/glossary/Operation/Sequence/$$
-for each test repetition.
-If it is false,
-the AD package is allowed to use one taping of the operation
+CppAD will use one taping of the operation
 sequence for all the repetitions of that speed test.
+Otherwise, the
+$cref/operation sequence/glossary/Operation/Sequence/$$
+will be retaped for each test repetition.
 $pre
 
 $$
 All of the tests, except $cref/det_lu/link_det_lu/$$,
 have a fixed operations sequence.
-The operation sequence for $code det_lu$$ 
+The operation sequence for $code det_lu$$
 may be different for each repetition of the test because it
-depends on the matrix for which the determinant is being calculated. 
-For this reason,
-$cref cppad_det_lu.cpp$$ returns false 
-(for test not implemented)
-when $code global_retape$$ is false.
+depends on the matrix for which the determinant is being calculated.
+For this reason, the CppAD test
+$cref cppad_det_lu.cpp$$ returns false
+(indicating that the test not implemented)
+when $code global_onetape$$ is true.
 
 $subhead optimize$$
-If the option $code optimize$$ is present, the symbol
+If the option $code optimize$$ is present,
 $codep
-	extern bool global_optimize
+	global_option["optimize"]
 $$
 is true and otherwise it is false.
 If this external symbol is true,
-and the AD package has an optional way to spend time optimizing
-the operation sequence,
-this optimization should be done before doing computations.
-If it is false,
-this optimization should not be done.
+CppAD will optimize the operation sequence before doing computations.
+If it is false, this optimization will not be done.
 
 $subhead atomic$$
-If the option $code atomic$$ is present, the symbol
+If the option $code atomic$$ is present,
 $codep
-	extern bool global_atomic
+	global_option["atomic"]
 $$
 is true and otherwise it is false.
-If this external symbol is true,
-and the AD package has a way to speed up the processing
-by adding $cref old_atomic$$ operations,
-this should be included in computations.
-If it is false, user defined atomic operations should not be done.
+If this external symbol is true, CppAD will use its user defined
+$cref/atomic/atomic_base/$$ operation is used for the test.
+If no such atomic operation exists,
+and atomic is chosen, CppAD returns false for the test.
 
 $subhead memory$$
-If the option $code memory$$ is present, the symbol
+If the option $code memory$$ is present,
 $codep
-	extern bool global_memory
+	global_option["memory"]
 $$
 is true and otherwise it is false.
 If it is true, the CppAD
-$cref/hold_memory/ta_hold_memory/$$ routine will be called by 
+$cref/hold_memory/ta_hold_memory/$$ routine will be called by
 the main program before any of the tests are executed.
 This should make the CppAD $code thread_alloc$$ allocator faster.
-If it is false, standard memory allocation should be done by.
-Otherwise the test can use special memory allocation to try 
-and improve speed.
+If it is false, CppAD will used standard memory allocation.
+
+$head Sparsity Options$$
+The following options only apply to the
+$cref/sparse_jacobian/link_sparse_jacobian/$$ and
+$cref/sparse_hessian/link_sparse_hessian/$$ tests.
+The other tests will ignore these options:
+
+$subhead boolsparsity$$
+If the option $code boolsparsity$$ is present,
+$codep
+	global_option["boolsparsity"]
+$$
+is true and otherwise it is false.
+If it is true, CppAD will use a
+$cref/vector of bool/glossary/Sparsity Pattern/Vector of Boolean/$$
+for its sparsity patterns.
+Otherwise it will use a
+$cref/vector of sets/glossary/Sparsity Pattern/Vector of Sets/$$.
+
+$subhead revsparsity$$
+If the option $code revsparsity$$ is present,
+$codep
+	global_option["revsparsity"]
+$$
+is true and otherwise it is false.
+If it is true, CppAD will use a
+$cref ForSparseJac$$ and $cref ForSparseHes$$ where possible
+for its sparsity patterns.
+Otherwise it will use $cref RevSparseJac$$ and $cref RevSparseHes$$.
+
+$subhead colpack$$
+If the option $code colpack$$ is present,
+$codep
+	global_option["colpack"]
+$$
+is true and otherwise it is false.
+If this external symbol is true,
+CppAD will use $cref/colpack/colpack_prefix/$$ to do the coloring
+for its
+Otherwise, it will use it's own coloring algorithm.
 
 $head Correctness Results$$
-An output line of the following form:
+One, but not both, of the following two output lines
 $codei%
+	%package%_%test%_%optionlist%_available = false
 	%package%_%test%_%optionlist%_ok = %flag%
 %$$
 is generated for each correctness test where
 $icode package$$ and $icode test$$ are as above,
-$icode optionlist$$ are the options (in $icode option_list$$) 
+$icode optionlist$$ are the options (in $icode option_list$$)
 separated by the underbar $code _$$ character
 (whereas they are separated by spaces in $icode option_list$$),
 and $icode flag$$ is $code true$$ or $code false$$.
@@ -222,8 +274,8 @@ For each speed test, corresponds to three lines of the
 following form are generated:
 $codei%
 	%package%_%test%_%optionlist%_ok   = %flag%
-	%package%_%test%_%optionlist%_size = [ %size_1%, %...%, %size_n% ]
-	%package%_%test%_%optionlist%_rate = [ %rate_1%, %...%, %rate_n% ]
+	%package%_%test%_size = [ %size_1%, %...%, %size_n% ]
+	%package%_%test%_rate = [ %rate_1%, %...%, %rate_n% ]
 %$$
 The values $icode package$$, $icode test$$, $icode optionlist$$,
 and $icode flag$$ are as in the correctness results above.
@@ -232,6 +284,21 @@ size arguments used for the corresponding tests.
 The values $icode rate_1$$, ..., $icode rate_n$$ are the number of times
 per second that the corresponding size problem executed.
 
+$subhead n_sweep$$
+The $cref/sparse_jacobian/link_sparse_jacobian/$$
+and $cref/sparse_hessian/link_sparse_hessian/$$ tests has an extra output
+line with the following form
+$codei%
+	%package%_sparse_%test%_n_sweep = [ %n_sweep_1%, %...%, %n_sweep_n% ]
+%$$
+were $icode test$$ is $code jacobian$$ ($code hessian$$).
+The values $icode n_sweep_1$$, ..., $icode n_sweep_n$$ are the number of
+sweeps (colors) used for each sparse Jacobian (Hessian) calculation; see
+$icode n_sweep$$ for
+$cref/sparse_jacobian/sparse_jacobian/n_sweep/$$ and
+$cref/sparse_hessian/sparse_hessian/n_sweep/$$.
+
+
 $children%
 	speed/src/link_det_lu.cpp%
 	speed/src/link_det_minor.cpp%
@@ -244,8 +311,9 @@ $children%
 %$$
 
 $head Link Functions$$
-Each speed test defines it's own version of one of the following 
-functions that link the speed test to the main program described above:
+Each $cref/package/speed_main/package/$$
+defines it's own version of one of the link functions listed below.
+Each of these functions links this main program to the corresponding test:
 $table
 $rref link_det_lu$$
 $rref link_det_minor$$
@@ -257,9 +325,10 @@ $rref link_sparse_jacobian$$
 $tend
 
 
-$end 
+$end
 -----------------------------------------------------------------------------
 */
+// external routines
 
 # define CPPAD_DECLARE_SPEED(name)                       \
      extern bool available_##name(void);                 \
@@ -274,26 +343,38 @@ CPPAD_DECLARE_SPEED(poly);
 CPPAD_DECLARE_SPEED(sparse_hessian);
 CPPAD_DECLARE_SPEED(sparse_jacobian);
 
-bool   global_retape;
-bool   global_optimize;
-bool   global_atomic;
-bool   global_memory;
+// info is different for each test
+extern void info_sparse_jacobian(size_t size, size_t& n_sweep);
+extern void info_sparse_hessian(size_t size, size_t& n_sweep);
+
+// --------------------------------------------------------------------------
+std::map<std::string, bool> global_option;
+// --------------------------------------------------------------------------
+
 
 namespace {
 	using std::cout;
 	using std::endl;
+	const char* option_list[] = {
+		"onetape",
+		"optimize",
+		"atomic",
+		"memory",
+		"boolsparsity",
+		"revsparsity",
+		"colpack",
+	};
+	size_t num_option = sizeof(option_list) / sizeof( option_list[0] );
 	// ----------------------------------------------------------------
 	// not available test message
 	void not_available_message(const char* test_name)
 	{	cout << AD_PACKAGE << ": " << test_name;
 		cout << " is not availabe with " << endl;
-		cout << "global_retape = " << global_retape;
-		cout << ", global_optimize = " << global_optimize;
-		cout << ", global_atomic = " << global_atomic;
-		cout << ", global_memory = " << global_memory;
-		cout << endl;
+		for(size_t i = 0; i < num_option; i++)
+		{	std::string option = option_list[i];
+			cout << option << " = " << global_option[option] << endl;
+		}
 	}
-
 	// ------------------------------------------------------
 	// output vector in form readable by octave or matlab
 	// convert size_t to int to avoid warning by MS compiler
@@ -312,22 +393,30 @@ namespace {
 	// function that runs one correctness case
 	static size_t Run_ok_count    = 0;
 	static size_t Run_error_count = 0;
-	bool run_correct(bool correct_case(bool), const char *case_name)
-	{	bool ok;
-# ifdef SPEED_DOUBLE
-		ok = correct_case(true);
+	bool run_correct(
+		bool available_case(void) ,
+		bool correct_case(bool)   ,
+		const char *case_name     )
+	{	bool available = available_case();
+		bool ok        = true;
+		if( available )
+		{
+# ifdef CPPAD_DOUBLE_SPEED
+			ok = correct_case(true);
 # else
-		ok = correct_case(false);
+			ok = correct_case(false);
 # endif
+		}
 		cout << AD_PACKAGE << "_" << case_name;
-		if( global_retape )
-			cout << "_retape";
-		if( global_optimize )
-			cout << "_optimize";
-		if( global_atomic )
-			cout << "_atomic";
-		if( global_memory )
-			cout << "_memory";
+		for(size_t i = 0; i < num_option; i++)
+		{	std::string option = option_list[i];
+			if( global_option[option]  )
+				cout << "_" << option;
+		}
+		if( ! available )
+		{	cout << "_available = false" << endl;
+			return ok;
+		}
 		cout << "_ok = ";
 		if( ok )
 		{	cout << " true" << endl;
@@ -342,29 +431,19 @@ namespace {
 	// ----------------------------------------------------------------
 	// function that runs one speed case
 	void run_speed(
-		void speed_case(size_t size, size_t repeat) , 
+		void speed_case(size_t size, size_t repeat) ,
 		const CppAD::vector<size_t>&       size_vec ,
 		const std::string&                case_name )
 	{	double time_min = 1.;
 		cout << AD_PACKAGE << "_" << case_name << "_size = ";
 		output(size_vec);
 		cout << endl;
-		cout << AD_PACKAGE << "_" << case_name;
-		if( global_retape )
-			cout << "_retape";
-		if( global_optimize )
-			cout << "_optimize";
-		if( global_atomic )
-			cout << "_atomic";
-		if( global_memory )
-			cout << "_memory";
-		cout << "_rate = ";
-
+		cout << AD_PACKAGE << "_" << case_name << "_rate = ";
 		cout << std::fixed;
 		for(size_t i = 0; i < size_vec.size(); i++)
 		{	if( i == 0 )
 				cout << "[ ";
-			else	cout << ", ";	
+			else	cout << ", ";
 			cout << std::flush;
 			size_t size = size_vec[i];
 			double time = CppAD::time_test(speed_case, time_min, size);
@@ -374,6 +453,9 @@ namespace {
 			else cout << std::setprecision(2) << rate;
 		}
 		cout << " ]" << endl;
+		//
+		// free statically allocated memory (size = repeat = 0)
+		speed_case(0, 0);
 		return;
 	}
 }
@@ -396,9 +478,9 @@ int main(int argc, char *argv[])
 	struct test_struct {
 		const char       *name;
 		const test_enum  index;
-	}; 
+	};
 	const test_struct test_list[]= {
-		{ "correct",            test_correct         }, 
+		{ "correct",            test_correct         },
 		{ "speed",              test_speed           },
 		{ "det_lu",             test_det_lu          },
 		{ "det_minor",          test_det_minor       },
@@ -410,50 +492,42 @@ int main(int argc, char *argv[])
 	};
 	const size_t n_test  = sizeof(test_list) / sizeof(test_list[0]);
 
-	size_t i;
 	test_enum match = test_error;
 	int    iseed = 0;
 	bool   error = argc < 3;
 	if( ! error )
-	{	for(i = 0; i < n_test; i++)
+	{	for(size_t i = 0; i < n_test; i++)
 			if( strcmp(test_list[i].name, argv[1]) == 0 )
 				match = test_list[i].index;
 		error = match == test_error;
 		iseed = std::atoi( argv[2] );
 		error |= iseed < 0;
-		global_retape   = false;
-		global_optimize = false;
-		global_atomic   = false;
-		global_memory   = false;
-		for(i = 3; i < size_t(argc); i++)
-		{	if( strcmp(argv[i], "retape") == 0 )
-				global_retape = true;
-			else if( strcmp(argv[i], "optimize") == 0 )
-				global_optimize = true;
-			else if( strcmp(argv[i], "atomic") == 0 )
-				global_atomic = true;
-			else if( strcmp(argv[i], "memory") == 0 )
-				global_memory = true;
-			else
-				error = true;
+		for(size_t i = 0; i < num_option; i++)
+			global_option[ option_list[i] ] = false;
+		for(size_t i = 3; i < size_t(argc); i++)
+		{	error = true;
+			for(size_t j = 0; j < num_option; j++)
+			{	if( strcmp(argv[i], option_list[j]) == 0 )
+				{	global_option[ option_list[j] ] = true;
+					error = false;
+				}
+			}
 		}
 	}
 	if( error )
-	{	cout << "usage: ./" 
+	{	cout << "usage: ./speed_"
 		     << AD_PACKAGE << " test seed option_list" << endl;
 		cout << "test choices: " << endl;
-		for(i = 0; i < n_test; i++)
+		for(size_t i = 0; i < n_test; i++)
 			cout << "\t" << test_list[i].name << endl;
-		cout << "seed choices: ";
-		cout << "a positive integer used as a random seed." << endl;
-		cout << "option choices: ";
-		cout << " \"retape\",";
-		cout << " \"optimize\",";
-		cout << " \"atomic\",";
-		cout << " \"memory\"." << endl << endl;
+		cout << "seed: is a positive integer used as a random seed." << endl;
+		cout << "option: is zero, one or more of the following:" << endl;
+		for(size_t i = 0; i < num_option; i++)
+			cout << " " << option_list[i];
+		cout << endl << endl;
 		return 1;
 	}
-	if( global_memory )
+	if( global_option["memory"] )
 		CppAD::thread_alloc::hold_memory(true);
 
 	// initialize the random number simulator
@@ -468,54 +542,56 @@ int main(int argc, char *argv[])
 	CppAD::vector<size_t> size_poly(n_size);
 	CppAD::vector<size_t> size_sparse_hessian(n_size);
 	CppAD::vector<size_t> size_sparse_jacobian(n_size);
-	for(i = 0; i < n_size; i++) 
-	{	size_det_lu[i]      = 3 * i + 1;
-		size_det_minor[i]   = i + 1;
-		size_mat_mul[i]     = 10 * (i + 1);
-		size_ode[i]         = 3 * i + 1;
-		size_poly[i]        = 8 * i + 1;
-		size_sparse_hessian[i]  = 100 * (i + 1) * (i + 1);
-		size_sparse_jacobian[i] = 100 * (i + 1) * (i + 1);
+	for(size_t i = 0; i < n_size; i++)
+	{	size_det_minor[i]   = i + 1;
+		size_det_lu[i]      = 10 * i + 1;
+		size_mat_mul[i]     = 10 * i + 1;
+		size_ode[i]         = 10 * i + 1;
+		size_poly[i]        = 10 * i + 1;
+		size_sparse_hessian[i]  = 150 * (i + 1) * (i + 1);
+		size_sparse_jacobian[i] = 150 * (i + 1) * (i + 1);
 	}
 
 	switch(match)
 	{
 		// run all the correctness tests
 		case test_correct:
-		if( available_det_lu() ) ok &= run_correct(
-			correct_det_lu,           "det_lu"       
+		ok &= run_correct( available_det_lu, correct_det_lu, "det_lu"
 		);
-		if( available_det_minor() ) ok &= run_correct(
-			correct_det_minor,        "det_minor"    
+		ok &= run_correct(
+			available_det_minor, correct_det_minor, "det_minor"
 		);
-		if( available_mat_mul() ) ok &= run_correct(
-			correct_mat_mul,          "mat_mul"    
+		ok &= run_correct(
+			available_mat_mul, correct_mat_mul, "mat_mul"
 		);
-		if( available_ode() ) ok &= run_correct(
-			correct_ode,             "ode"         
+		ok &= run_correct(
+			available_ode, correct_ode, "ode"
 		);
-		if( available_poly() ) ok &= run_correct(
-			correct_poly,            "poly"         
+		ok &= run_correct( available_poly, correct_poly, "poly"
 		);
-		if( available_sparse_hessian() ) ok &= run_correct(
-			correct_sparse_hessian,  "sparse_hessian"         
+		ok &= run_correct(
+			available_sparse_hessian,
+			correct_sparse_hessian,
+			"sparse_hessian"
 		);
-		if( available_sparse_jacobian() ) ok &= run_correct(
-			correct_sparse_jacobian, "sparse_jacobian"         
+		ok &= run_correct(
+			available_sparse_jacobian,
+			correct_sparse_jacobian,
+			"sparse_jacobian"
 		);
 		// summarize results
 		assert( ok || (Run_error_count > 0) );
 		if( ok )
-		{	cout	<< "All " << int(Run_ok_count) 
+		{	cout	<< "All " << int(Run_ok_count)
 				<< " correctness tests passed." << endl;
 		}
 		else
-		{	cout	<< int(Run_error_count) 
+		{	cout	<< int(Run_error_count)
 				<< " correctness tests failed." << endl;
 		}
 		break;
 		// ---------------------------------------------------------
-		// run all the speed tests 
+		// run all the speed tests
 		case test_speed:
 		if( available_det_lu() ) run_speed(
 		speed_det_lu,          size_det_lu,          "det_lu"
@@ -544,77 +620,116 @@ int main(int argc, char *argv[])
 
 		case test_det_lu:
 		if( ! available_det_lu() )
-		{	not_available_message( argv[1] ); 
+		{	not_available_message( argv[1] );
 			exit(1);
 		}
-		ok &= run_correct(correct_det_lu,           "det_lu");
+		ok &= run_correct(
+			available_det_lu, correct_det_lu, "det_lu")
+		;
 		run_speed(speed_det_lu,    size_det_lu,     "det_lu");
 		break;
 		// ---------------------------------------------------------
 
 		case test_det_minor:
 		if( ! available_det_minor() )
-		{	not_available_message( argv[1] ); 
+		{	not_available_message( argv[1] );
 			exit(1);
 		}
-		ok &= run_correct(correct_det_minor,       "det_minor");
+		ok &= run_correct(
+			available_det_minor, correct_det_minor, "det_minor"
+		);
 		run_speed(speed_det_minor, size_det_minor, "det_minor");
 		break;
 		// ---------------------------------------------------------
 
 		case test_mat_mul:
 		if( ! available_mat_mul() )
-		{	not_available_message( argv[1] ); 
+		{	not_available_message( argv[1] );
 			exit(1);
 		}
-		ok &= run_correct(correct_mat_mul,     "mat_mul");
+		ok &= run_correct(
+			available_mat_mul, correct_mat_mul, "mat_mul"
+		);
 		run_speed(speed_mat_mul, size_mat_mul, "mat_mul");
 		break;
 		// ---------------------------------------------------------
 
 		case test_ode:
 		if( ! available_ode() )
-		{	not_available_message( argv[1] ); 
+		{	not_available_message( argv[1] );
 			exit(1);
 		}
-		ok &= run_correct(correct_ode,           "ode");
+		ok &= run_correct(
+			available_ode, correct_ode, "ode"
+		);
 		run_speed(speed_ode,      size_ode,      "ode");
 		break;
 		// ---------------------------------------------------------
 
 		case test_poly:
 		if( ! available_poly() )
-		{	not_available_message( argv[1] ); 
+		{	not_available_message( argv[1] );
 			exit(1);
 		}
-		ok &= run_correct(correct_poly,            "poly");
+		ok &= run_correct(
+			available_poly, correct_poly, "poly"
+		);
 		run_speed(speed_poly,      size_poly,      "poly");
 		break;
 		// ---------------------------------------------------------
 
 		case test_sparse_hessian:
 		if( ! available_sparse_hessian() )
-		{	not_available_message( argv[1] ); 
+		{	not_available_message( argv[1] );
 			exit(1);
 		}
-		ok &= run_correct(correct_sparse_hessian, "sparse_hessian");
+		ok &= run_correct(
+			available_sparse_hessian,
+			correct_sparse_hessian,
+			"sparse_hessian"
+		);
 		run_speed(
-		speed_sparse_hessian, size_sparse_hessian,  "sparse_hessian");
+			speed_sparse_hessian, size_sparse_hessian,  "sparse_hessian"
+		);
+		cout << AD_PACKAGE << "_sparse_hessian_sweep = ";
+		for(size_t i = 0; i < size_sparse_hessian.size(); i++)
+		{	if( i == 0 )
+				cout << "[ ";
+			else	cout << ", ";
+			size_t n_sweep;
+			info_sparse_hessian(size_sparse_hessian[i], n_sweep);
+			cout << n_sweep;
+		}
+		cout << " ]" << endl;
 		break;
 		// ---------------------------------------------------------
 
 		case test_sparse_jacobian:
 		if( ! available_sparse_jacobian() )
-		{	not_available_message( argv[1] ); 
+		{	not_available_message( argv[1] );
 			exit(1);
 		}
-		ok &= run_correct(correct_sparse_jacobian, "sparse_jacobian");
-		run_speed(
-		speed_sparse_jacobian, size_sparse_jacobian, "sparse_jacobian"
+		ok &= run_correct(
+			available_sparse_jacobian,
+			correct_sparse_jacobian,
+			"sparse_jacobian"
 		);
+		run_speed(
+			speed_sparse_jacobian, size_sparse_jacobian, "sparse_jacobian"
+		);
+		cout << AD_PACKAGE << "_sparse_jacobian_n_sweep = ";
+		for(size_t i = 0; i < size_sparse_jacobian.size(); i++)
+		{	if( i == 0 )
+				cout << "[ ";
+			else	cout << ", ";
+			size_t n_sweep;
+			info_sparse_jacobian(size_sparse_jacobian[i], n_sweep);
+			cout << n_sweep;
+		}
+		cout << " ]" << endl;
 		break;
 		// ---------------------------------------------------------
-		
+
 		default:
 		assert(0);
 	}
