@@ -1,19 +1,19 @@
-/* $Id: extern_value.hpp 3275 2014-05-20 17:35:11Z bradbell $ */
+// $Id: extern_value.hpp 3804 2016-03-20 15:08:46Z bradbell $
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-14 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
-the terms of the 
+the terms of the
                     Eclipse Public License Version 1.0.
 
 A copy of this license is included in the COPYING file of this distribution.
 Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 -------------------------------------------------------------------------- */
 
-# ifndef CPPAD_EXTERN_VALUE_INCLUDED
-# define CPPAD_EXTERN_VALUE_INCLUDED
+# ifndef CPPAD_TEST_MORE_EXTERN_VALUE_HPP
+# define CPPAD_TEST_MORE_EXTERN_VALUE_HPP
 
-template <class Type> 
+template <class Type>
 class extern_value  {
 private:
 	Type value_;
